@@ -3,6 +3,8 @@
  * opened cursors and kills them on cleanup.
  */
 
+load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
+
 /**
  * Enumeration of the possible types of change streams.
  */
@@ -42,15 +44,46 @@ const runCommandChangeStreamPassthroughAware =
  *     - single collection streams: drop, rename, and dropDatabase.
  *     - whole DB streams: dropDatabase.
  *     - whole cluster streams: none.
+ * Returns the invalidate document if there was one, or null otherwise.
  */
 function assertInvalidateOp({cursor, opType}) {
     if (!isChangeStreamPassthrough() ||
         (changeStreamPassthroughType() == ChangeStreamWatchMode.kDb && opType == "dropDatabase")) {
         assert.soon(() => cursor.hasNext());
-        assert.eq(cursor.next().operationType, "invalidate");
+        const invalidate = cursor.next();
+        assert.eq(invalidate.operationType, "invalidate");
         assert(cursor.isExhausted());
         assert(cursor.isClosed());
+        return invalidate;
     }
+    return null;
+}
+
+/**
+ * Helper to check whether a change stream event matches the given expected event. Ignores the
+ * resume token and clusterTime unless they are explicitly listed in the expectedEvent.
+ */
+function assertChangeStreamEventEq(actualEvent, expectedEvent) {
+    const testEvent = Object.assign({}, actualEvent);
+    if (!expectedEvent.hasOwnProperty("_id")) {
+        delete testEvent._id;  // Remove the resume token, if present.
+    }
+    if (!expectedEvent.hasOwnProperty("clusterTime")) {
+        delete testEvent.clusterTime;  // Remove the cluster time, if present.
+    }
+
+    // The change stream transaction passthrough causes operations to have txnNumber and lsid
+    // values that the test doesn't expect, which can cause comparisons to fail.
+    if (!expectedEvent.hasOwnProperty("txnNumber")) {
+        delete testEvent.txnNumber;  // Remove the txnNumber, if present.
+    }
+    if (!expectedEvent.hasOwnProperty("lsid")) {
+        delete testEvent.lsid;  // Remove the lsid, if present.
+    }
+    assert.docEq(testEvent,
+                 expectedEvent,
+                 "Change did not match expected change. Expected change: " + tojson(expectedEvent) +
+                     ", Actual change: " + tojson(testEvent));
 }
 
 function ChangeStreamTest(_db, name = "ChangeStreamTest") {
@@ -154,17 +187,8 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
     function assertChangeIsExpected(
         expectedChanges, numChangesSeen, observedChanges, expectInvalidate) {
         if (expectedChanges) {
-            const lastObservedChange = Object.assign({}, observedChanges[numChangesSeen]);
-            if (expectedChanges[numChangesSeen]._id == null) {
-                delete lastObservedChange._id;  // Remove the resume token, if present.
-            }
-            if (expectedChanges[numChangesSeen].clusterTime == null) {
-                delete lastObservedChange.clusterTime;  // Remove the cluster time, if present.
-            }
-            assert.docEq(lastObservedChange,
-                         expectedChanges[numChangesSeen],
-                         "Change did not match expected change. Expected changes: " +
-                             tojson(expectedChanges));
+            assertChangeStreamEventEq(observedChanges[numChangesSeen],
+                                      expectedChanges[numChangesSeen]);
         } else if (!expectInvalidate) {
             assert(!isInvalidated(observedChanges[numChangesSeen]),
                    "Change was invalidated when it should not have been. Number of changes seen: " +
@@ -281,7 +305,6 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
                 }));
             }
         }
-
     };
 
     /**
