@@ -116,6 +116,9 @@ ReplSetTest.prototype.stepdown = function(nodeId) {
         return true;
     });
 
+    // Wait for the state to be secondary to avoid the race of closing new connections on stepdown
+    // on 3.2 and 3.4.
+    this.waitForState(node, ReplSetTest.State.SECONDARY);
     return this.reconnect(node);
 };
 
