@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -33,7 +32,6 @@
 #include <memory>
 #include <string>
 
-#include "mongo/base/disallow_copying.h"
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
 #include "mongo/stdx/functional.h"
@@ -54,7 +52,8 @@ namespace mongo {
  * parameterPassword parameter is not constrained.
  */
 class SaslClientSession {
-    MONGO_DISALLOW_COPYING(SaslClientSession);
+    SaslClientSession(const SaslClientSession&) = delete;
+    SaslClientSession& operator=(const SaslClientSession&) = delete;
 
 public:
     typedef stdx::function<SaslClientSession*(const std::string&)> SaslClientSessionFactoryFn;
