@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -35,7 +34,7 @@
 #include "mongo/base/init.h"
 #include "mongo/base/status.h"
 #include "mongo/bson/util/builder.h"
-#include "mongo/client/dbclientinterface.h"
+#include "mongo/client/dbclient_connection.h"
 #include "mongo/db/auth/action_set.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/resource_pattern.h"
@@ -43,10 +42,7 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/cloner.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/commands/copydb.h"
 #include "mongo/db/commands/rename_collection.h"
-#include "mongo/db/db.h"
-#include "mongo/db/index_builder.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/insert.h"
@@ -56,11 +52,15 @@
 
 namespace mongo {
 
-using std::unique_ptr;
+using std::endl;
 using std::string;
 using std::stringstream;
-using std::endl;
+using std::unique_ptr;
 
+/**
+ * The cloneCollection command is deprecated.
+ * See http://dochub.mongodb.org/core/clonecollection-deprecation.
+ */
 class CmdCloneCollection : public ErrmsgCommandDeprecated {
 public:
     CmdCloneCollection() : ErrmsgCommandDeprecated("cloneCollection") {}
@@ -109,6 +109,12 @@ public:
                            const BSONObj& cmdObj,
                            string& errmsg,
                            BSONObjBuilder& result) {
+        const char* deprecationWarning =
+            "Support for the cloneCollection command has been deprecated. See "
+            "http://dochub.mongodb.org/core/clonecollection-deprecation";
+        warning() << deprecationWarning;
+        result.append("note", deprecationWarning);
+
         boost::optional<DisableDocumentValidation> maybeDisableValidation;
         if (shouldBypassDocumentValidationForCommand(cmdObj))
             maybeDisableValidation.emplace(opCtx);
@@ -131,9 +137,8 @@ public:
 
         // In order to clone a namespace, a user must be allowed to both create and write to that
         // namespace. There exist namespaces that are legal to create but not write to (e.g.
-        // system.profile), and there exist namespaces that are legal to write to but not create
-        // (e.g. system.indexes), so we must check that it is legal to both create and write to the
-        // namespace.
+        // system.profile), and there exist namespaces that are legal to write to but not create,
+        // so we must check that it is legal to both create and write to the namespace.
         auto allowedCreateStatus = userAllowedCreateNS(dbname, nsToCollectionSubstring(ns));
         uassertStatusOK(allowedCreateStatus);
         auto allowedWriteStatus = userAllowedWriteNS(dbname, nsToCollectionSubstring(ns));
