@@ -381,7 +381,7 @@ public:
                     statsTracker.emplace(opCtx,
                                          _request.nss,
                                          Top::LockType::NotLocked,
-                                         AutoStatsTracker::LogMode::kUpdateTopAndCurop,
+                                         AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
                                          dbProfilingLevel);
                 }
             } else {
@@ -411,7 +411,7 @@ public:
                 statsTracker.emplace(opCtx,
                                      _request.nss,
                                      Top::LockType::ReadLocked,
-                                     AutoStatsTracker::LogMode::kUpdateTopAndCurop,
+                                     AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
                                      readLock->getDb() ? readLock->getDb()->getProfilingLevel()
                                                        : doNotChangeProfilingLevel);
 
