@@ -119,6 +119,7 @@
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/config_server_op_observer.h"
 #include "mongo/db/s/op_observer_sharding_impl.h"
+#include "mongo/db/s/periodic_sharded_index_consistency_checker.h"
 #include "mongo/db/s/shard_server_op_observer.h"
 #include "mongo/db/s/sharding_initialization_mongod.h"
 #include "mongo/db/s/sharding_state_recovery.h"
@@ -918,17 +919,21 @@ void shutdownTask(const ShutdownTaskArgs& shutdownArgs) {
             opCtx = uniqueOpCtx.get();
         }
 
-        try {
-            // For faster tests, we allow a short wait time with setParameter.
-            auto waitTime = repl::waitForStepDownOnNonCommandShutdown.load()
-                ? Milliseconds(Seconds(10))
-                : Milliseconds(100);
-
-            replCoord->stepDown(opCtx, false /* force */, waitTime, Seconds(120));
-        } catch (const ExceptionFor<ErrorCodes::NotMaster>&) {
-            // ignore not master errors
-        } catch (const DBException& e) {
-            log() << "Failed to stepDown in non-command initiated shutdown path " << e.toString();
+        // If this is a single node replica set, then we don't have to wait
+        // for any secondaries. Ignore stepdown.
+        if (repl::ReplicationCoordinator::get(serviceContext)->getConfig().getNumMembers() != 1) {
+            try {
+                // For faster tests, we allow a short wait time with setParameter.
+                auto waitTime = repl::waitForStepDownOnNonCommandShutdown.load()
+                    ? Milliseconds(Seconds(10))
+                    : Milliseconds(100);
+                replCoord->stepDown(opCtx, false /* force */, waitTime, Seconds(120));
+            } catch (const ExceptionFor<ErrorCodes::NotMaster>&) {
+                // ignore not master errors
+            } catch (const DBException& e) {
+                log() << "Failed to stepDown in non-command initiated shutdown path "
+                      << e.toString();
+            }
         }
     }
 
@@ -945,6 +950,11 @@ void shutdownTask(const ShutdownTaskArgs& shutdownArgs) {
         lsc->joinOnShutDown();
     }
 
+    // Terminate the index consistency check.
+    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
+        PeriodicShardedIndexConsistencyChecker::get(serviceContext).onShutDown();
+    }
+
     // Shutdown the TransportLayer so that new connections aren't accepted
     if (auto tl = serviceContext->getTransportLayer()) {
         log(LogComponent::kNetwork) << "shutdown: going to close listening sockets...";
@@ -1012,6 +1022,10 @@ void shutdownTask(const ShutdownTaskArgs& shutdownArgs) {
         validator->shutDown();
     }
 
+    if (ShardingState::get(serviceContext)->enabled()) {
+        CatalogCacheLoader::get(serviceContext).shutDown();
+    }
+
 #if __has_feature(address_sanitizer)
     // When running under address sanitizer, we get false positive leaks due to disorder around
     // the lifecycle of a connection and request. When we are running under ASAN, we try a lot
