@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -40,7 +39,7 @@
 #include "mongo/db/exec/scoped_timer.h"
 #include "mongo/db/exec/working_set_computed_data.h"
 #include "mongo/db/index/index_access_method.h"
-#include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/index_names.h"
 #include "mongo/db/query/index_bounds_builder.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/log.h"
@@ -62,53 +61,44 @@ namespace mongo {
 const char* IndexScan::kStageType = "IXSCAN";
 
 IndexScan::IndexScan(OperationContext* opCtx,
-                     const IndexScanParams& params,
+                     IndexScanParams params,
                      WorkingSet* workingSet,
                      const MatchExpression* filter)
-    : PlanStage(kStageType, opCtx),
+    : RequiresIndexStage(kStageType, opCtx, params.indexDescriptor),
       _workingSet(workingSet),
-      _iam(params.descriptor->getIndexCatalog()->getIndex(params.descriptor)),
-      _keyPattern(params.descriptor->keyPattern().getOwned()),
-      _scanState(INITIALIZING),
+      _keyPattern(params.keyPattern.getOwned()),
+      _bounds(std::move(params.bounds)),
       _filter(filter),
-      _shouldDedup(true),
+      _direction(params.direction),
       _forward(params.direction == 1),
-      _params(params),
+      _shouldDedup(params.shouldDedup),
+      _addKeyMetadata(params.addKeyMetadata),
       _startKeyInclusive(IndexBounds::isStartIncludedInBound(params.bounds.boundInclusion)),
       _endKeyInclusive(IndexBounds::isEndIncludedInBound(params.bounds.boundInclusion)) {
-    // We can't always access the descriptor in the call to getStats() so we pull
-    // any info we need for stats reporting out here.
+    _specificStats.indexName = params.name;
     _specificStats.keyPattern = _keyPattern;
-    if (BSONElement collationElement = _params.descriptor->getInfoElement("collation")) {
-        invariant(collationElement.isABSONObj());
-        _specificStats.collation = collationElement.Obj().getOwned();
-    }
-    _specificStats.indexName = _params.descriptor->indexName();
-    _specificStats.isMultiKey = _params.descriptor->isMultikey(getOpCtx());
-    _specificStats.multiKeyPaths = _params.descriptor->getMultikeyPaths(getOpCtx());
-    _specificStats.isUnique = _params.descriptor->unique();
-    _specificStats.isSparse = _params.descriptor->isSparse();
-    _specificStats.isPartial = _params.descriptor->isPartial();
-    _specificStats.indexVersion = static_cast<int>(_params.descriptor->version());
+    _specificStats.isMultiKey = params.isMultiKey;
+    _specificStats.multiKeyPaths = params.multikeyPaths;
+    _specificStats.isUnique = params.indexDescriptor->unique();
+    _specificStats.isSparse = params.indexDescriptor->isSparse();
+    _specificStats.isPartial = params.indexDescriptor->isPartial();
+    _specificStats.indexVersion = static_cast<int>(params.indexDescriptor->version());
+    _specificStats.collation = params.indexDescriptor->infoObj()
+                                   .getObjectField(IndexDescriptor::kCollationFieldName)
+                                   .getOwned();
 }
 
 boost::optional<IndexKeyEntry> IndexScan::initIndexScan() {
-    if (_params.doNotDedup) {
-        _shouldDedup = false;
-    } else {
-        _shouldDedup = _params.descriptor->isMultikey(getOpCtx());
-    }
-
     // Perform the possibly heavy-duty initialization of the underlying index cursor.
-    _indexCursor = _iam->newCursor(getOpCtx(), _forward);
+    _indexCursor = indexAccessMethod()->newCursor(getOpCtx(), _forward);
 
     // We always seek once to establish the cursor position.
     ++_specificStats.seeks;
 
-    if (_params.bounds.isSimpleRange) {
+    if (_bounds.isSimpleRange) {
         // Start at one key, end at another.
-        _startKey = _params.bounds.startKey;
-        _endKey = _params.bounds.endKey;
+        _startKey = _bounds.startKey;
+        _endKey = _bounds.endKey;
         _indexCursor->setEndPosition(_endKey, _endKeyInclusive);
         return _indexCursor->seek(_startKey, _startKeyInclusive);
     } else {
@@ -116,11 +106,11 @@ boost::optional<IndexKeyEntry> IndexScan::initIndexScan() {
         // of an end cursor.  For all other index scans, we fall back on using
         // IndexBoundsChecker to determine when we've finished the scan.
         if (IndexBoundsBuilder::isSingleInterval(
-                _params.bounds, &_startKey, &_startKeyInclusive, &_endKey, &_endKeyInclusive)) {
+                _bounds, &_startKey, &_startKeyInclusive, &_endKey, &_endKeyInclusive)) {
             _indexCursor->setEndPosition(_endKey, _endKeyInclusive);
             return _indexCursor->seek(_startKey, _startKeyInclusive);
         } else {
-            _checker.reset(new IndexBoundsChecker(&_params.bounds, _keyPattern, _params.direction));
+            _checker.reset(new IndexBoundsChecker(&_bounds, _keyPattern, _direction));
 
             if (!_checker->getStartSeekPoint(&_seekPoint))
                 return boost::none;
@@ -157,7 +147,7 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
         // In debug mode, check that the cursor isn't lying to us.
         if (kDebugBuild && !_startKey.isEmpty()) {
             int cmp = kv->key.woCompare(_startKey,
-                                        Ordering::make(_params.descriptor->keyPattern()),
+                                        Ordering::make(_keyPattern),
                                         /*compareFieldNames*/ false);
             if (cmp == 0)
                 dassert(_startKeyInclusive);
@@ -166,7 +156,7 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
 
         if (kDebugBuild && !_endKey.isEmpty()) {
             int cmp = kv->key.woCompare(_endKey,
-                                        Ordering::make(_params.descriptor->keyPattern()),
+                                        Ordering::make(_keyPattern),
                                         /*compareFieldNames*/ false);
             if (cmp == 0)
                 dassert(_endKeyInclusive);
@@ -174,9 +164,6 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
         }
 
         ++_specificStats.keysExamined;
-        if (_params.maxScan && _specificStats.keysExamined >= _params.maxScan) {
-            kv = boost::none;
-        }
     }
 
     if (kv && _checker) {
@@ -225,13 +212,12 @@ PlanStage::StageState IndexScan::doWork(WorkingSetID* out) {
     WorkingSetID id = _workingSet->allocate();
     WorkingSetMember* member = _workingSet->get(id);
     member->recordId = kv->loc;
-    member->keyData.push_back(IndexKeyDatum(_keyPattern, kv->key, _iam));
+    member->keyData.push_back(IndexKeyDatum(_keyPattern, kv->key, indexAccessMethod()));
     _workingSet->transitionToRecordIdAndIdx(id);
 
-    if (_params.addKeyMetadata) {
-        BSONObjBuilder bob;
-        bob.appendKeys(_keyPattern, kv->key);
-        member->addComputed(new IndexKeyComputedData(bob.obj()));
+    if (_addKeyMetadata) {
+        member->addComputed(
+            new IndexKeyComputedData(IndexKeyComputedData::rehydrateKey(_keyPattern, kv->key)));
     }
 
     *out = id;
@@ -242,7 +228,7 @@ bool IndexScan::isEOF() {
     return _commonStats.isEOF;
 }
 
-void IndexScan::doSaveState() {
+void IndexScan::doSaveStateRequiresIndex() {
     if (!_indexCursor)
         return;
 
@@ -254,7 +240,7 @@ void IndexScan::doSaveState() {
     _indexCursor->save();
 }
 
-void IndexScan::doRestoreState() {
+void IndexScan::doRestoreStateRequiresIndex() {
     if (_indexCursor)
         _indexCursor->restore();
 }
@@ -269,22 +255,6 @@ void IndexScan::doReattachToOperationContext() {
         _indexCursor->reattachToOperationContext(getOpCtx());
 }
 
-void IndexScan::doInvalidate(OperationContext* opCtx, const RecordId& dl, InvalidationType type) {
-    // The only state we're responsible for holding is what RecordIds to drop.  If a document
-    // mutates the underlying index cursor will deal with it.
-    if (INVALIDATION_MUTATION == type) {
-        return;
-    }
-
-    // If we see this RecordId again, it may not be the same document it was before, so we want
-    // to return it if we see it again.
-    stdx::unordered_set<RecordId, RecordId::Hasher>::iterator it = _returned.find(dl);
-    if (it != _returned.end()) {
-        ++_specificStats.seenInvalidated;
-        _returned.erase(it);
-    }
-}
-
 std::unique_ptr<PlanStageStats> IndexScan::getStats() {
     // WARNING: this could be called even if the collection was dropped.  Do not access any
     // catalog information here.
@@ -300,9 +270,9 @@ std::unique_ptr<PlanStageStats> IndexScan::getStats() {
     if (_specificStats.indexType.empty()) {
         _specificStats.indexType = "BtreeCursor";  // TODO amName;
 
-        _specificStats.indexBounds = _params.bounds.toBSON();
+        _specificStats.indexBounds = _bounds.toBSON();
 
-        _specificStats.direction = _params.direction;
+        _specificStats.direction = _direction;
     }
 
     std::unique_ptr<PlanStageStats> ret =
