@@ -1,6 +1,3 @@
-// expression_parser.h
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -65,6 +62,8 @@ enum class PathAcceptingKeyword {
     GREATER_THAN_OR_EQUAL,
     INTERNAL_EXPR_EQ,
     INTERNAL_SCHEMA_ALL_ELEM_MATCH_FROM_INDEX,
+    INTERNAL_SCHEMA_BIN_DATA_ENCRYPTED_TYPE,
+    INTERNAL_SCHEMA_BIN_DATA_SUBTYPE,
     INTERNAL_SCHEMA_EQ,
     INTERNAL_SCHEMA_FMOD,
     INTERNAL_SCHEMA_MATCH_ARRAY_INDEX,
@@ -107,11 +106,6 @@ public:
     static constexpr AllowedFeatureSet kDefaultSpecialFeatures =
         AllowedFeatures::kExpr | AllowedFeatures::kJSONSchema;
 
-    /**
-     * Constant double representation of 2^63.
-     */
-    static const double kLongLongMaxPlusOneAsDouble;
-
     /**
      * Parses PathAcceptingKeyword from 'typeElem'. Returns 'defaultKeyword' if 'typeElem'
      * doesn't represent a known type, or represents PathAcceptingKeyword::EQUALITY which is not
@@ -129,35 +123,5 @@ public:
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const ExtensionsCallback& extensionsCallback = ExtensionsCallbackNoop(),
         AllowedFeatureSet allowedFeatures = kDefaultSpecialFeatures);
-
-    /**
-     * Parses a BSONElement of any numeric type into a positive long long, failing if the value
-     * is any of the following:
-     *
-     * - NaN.
-     * - Negative.
-     * - A floating point number which is not integral.
-     * - Too large to fit within a 64-bit signed integer.
-     */
-    static StatusWith<long long> parseIntegerElementToNonNegativeLong(BSONElement elem);
-
-    /**
-     * Parses a BSONElement of any numeric type into a long long, failing if the value
-     * is any of the following:
-     *
-     * - NaN.
-     * - A floating point number which is not integral.
-     * - Too large in the positive or negative direction to fit within a 64-bit signed integer.
-     */
-    static StatusWith<long long> parseIntegerElementToLong(BSONElement elem);
-
-    /**
-     * Parses a BSONElement of any numeric type into an integer, failing if the value is:
-     *
-     * - NaN
-     * - a non-integral number
-     * - too large in the positive or negative direction to fit in an int
-     */
-    static StatusWith<int> parseIntegerElementToInt(BSONElement elem);
 };
 }  // namespace mongo
