@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -66,11 +65,9 @@ StatusWith<boost::optional<StringData>> parseTopLevelFieldName(MatchExpression*
 
             if (statusWithId.getValue() && placeholder != statusWithId.getValue()) {
                 return Status(ErrorCodes::FailedToParse,
-                              str::stream() << "Expected a single top-level field name, found '"
-                                            << *placeholder
-                                            << "' and '"
-                                            << *statusWithId.getValue()
-                                            << "'");
+                              str::stream()
+                                  << "Expected a single top-level field name, found '"
+                                  << *placeholder << "' and '" << *statusWithId.getValue() << "'");
             }
         }
         return placeholder;
@@ -106,8 +103,7 @@ StatusWith<std::unique_ptr<ExpressionWithPlaceholder>> ExpressionWithPlaceholder
             return Status(ErrorCodes::BadValue,
                           str::stream() << "The top-level field name must be an alphanumeric "
                                            "string beginning with a lowercase letter, found '"
-                                        << *placeholder
-                                        << "'");
+                                        << *placeholder << "'");
         }
     }
 
