@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -59,23 +58,78 @@ namespace {
  * places, rather than compiled in one place and linked, and so cannot provide a globally unique ID.
  */
 std::string nextFileName() {
-    static AtomicUInt32 documentSourceGroupFileCounter;
+    static AtomicWord<unsigned> documentSourceGroupFileCounter;
     return "extsort-doc-group." + std::to_string(documentSourceGroupFileCounter.fetchAndAdd(1));
 }
 
 }  // namespace
 
 using boost::intrusive_ptr;
-using std::shared_ptr;
 using std::pair;
+using std::shared_ptr;
 using std::vector;
 
+Document GroupFromFirstDocumentTransformation::applyTransformation(const Document& input) {
+    MutableDocument output(_accumulatorExprs.size());
+
+    for (auto&& expr : _accumulatorExprs) {
+        auto value = expr.second->evaluate(input, &expr.second->getExpressionContext()->variables);
+        output.addField(expr.first, value.missing() ? Value(BSONNULL) : value);
+    }
+
+    return output.freeze();
+}
+
+void GroupFromFirstDocumentTransformation::optimize() {
+    for (auto&& expr : _accumulatorExprs) {
+        expr.second = expr.second->optimize();
+    }
+}
+
+Document GroupFromFirstDocumentTransformation::serializeTransformation(
+    boost::optional<ExplainOptions::Verbosity> explain) const {
+
+    MutableDocument newRoot(_accumulatorExprs.size());
+    for (auto&& expr : _accumulatorExprs) {
+        newRoot.addField(expr.first, expr.second->serialize(static_cast<bool>(explain)));
+    }
+
+    return {{"newRoot", newRoot.freezeToValue()}};
+}
+
+DepsTracker::State GroupFromFirstDocumentTransformation::addDependencies(DepsTracker* deps) const {
+    for (auto&& expr : _accumulatorExprs) {
+        expr.second->addDependencies(deps);
+    }
+
+    // This stage will replace the entire document with a new document, so any existing fields
+    // will be replaced and cannot be required as dependencies. We use EXHAUSTIVE_ALL here
+    // instead of EXHAUSTIVE_FIELDS, as in ReplaceRootTransformation, because the stages that
+    // follow a $group stage should not depend on document metadata.
+    return DepsTracker::State::EXHAUSTIVE_ALL;
+}
+
+DocumentSource::GetModPathsReturn GroupFromFirstDocumentTransformation::getModifiedPaths() const {
+    // Replaces the entire root, so all paths are modified.
+    return {DocumentSource::GetModPathsReturn::Type::kAllPaths, std::set<std::string>{}, {}};
+}
+
+std::unique_ptr<GroupFromFirstDocumentTransformation> GroupFromFirstDocumentTransformation::create(
+    const intrusive_ptr<ExpressionContext>& expCtx,
+    const std::string& groupId,
+    vector<pair<std::string, intrusive_ptr<Expression>>> accumulatorExprs) {
+    return std::make_unique<GroupFromFirstDocumentTransformation>(groupId,
+                                                                  std::move(accumulatorExprs));
+}
+
+constexpr StringData DocumentSourceGroup::kStageName;
+
 REGISTER_DOCUMENT_SOURCE(group,
                          LiteParsedDocumentSourceDefault::parse,
                          DocumentSourceGroup::createFromBson);
 
 const char* DocumentSourceGroup::getSourceName() const {
-    return "$group";
+    return kStageName.rawData();
 }
 
 DocumentSource::GetNextResult DocumentSourceGroup::getNext() {
@@ -95,8 +149,6 @@ DocumentSource::GetNextResult DocumentSourceGroup::getNext() {
 
     if (_spilled) {
         return getNextSpilled();
-    } else if (_streaming) {
-        return getNextStreaming();
     } else {
         return getNextStandard();
     }
@@ -149,44 +201,6 @@ DocumentSource::GetNextResult DocumentSourceGroup::getNextStandard() {
     return std::move(out);
 }
 
-DocumentSource::GetNextResult DocumentSourceGroup::getNextStreaming() {
-    // Streaming optimization is active.
-    if (!_firstDocOfNextGroup) {
-        auto nextInput = pSource->getNext();
-        if (!nextInput.isAdvanced()) {
-            return nextInput;
-        }
-        _firstDocOfNextGroup = nextInput.releaseDocument();
-    }
-
-    Value id;
-    do {
-        // Add to the current accumulator(s).
-        for (size_t i = 0; i < _currentAccumulators.size(); i++) {
-            _currentAccumulators[i]->process(
-                _accumulatedFields[i].expression->evaluate(*_firstDocOfNextGroup), _doingMerge);
-        }
-
-        // Retrieve the next document.
-        auto nextInput = pSource->getNext();
-        if (!nextInput.isAdvanced()) {
-            return nextInput;
-        }
-
-        _firstDocOfNextGroup = nextInput.releaseDocument();
-
-
-        // Compute the id. If it does not match _currentId, we will exit the loop, leaving
-        // _firstDocOfNextGroup set for the next time getNext() is called.
-        id = computeId(*_firstDocOfNextGroup);
-    } while (pExpCtx->getValueComparator().evaluate(_currentId == id));
-
-    Document out = makeDocument(_currentId, _currentAccumulators, pExpCtx->needsMerge);
-    _currentId = std::move(id);
-
-    return std::move(out);
-}
-
 void DocumentSourceGroup::doDispose() {
     // Free our resources.
     _groups = pExpCtx->getValueComparator().makeUnorderedValueMap<Accumulators>();
@@ -194,8 +208,6 @@ void DocumentSourceGroup::doDispose() {
 
     // Make us look done.
     groupsIterator = _groups->end();
-
-    _firstDocOfNextGroup = boost::none;
 }
 
 intrusive_ptr<DocumentSource> DocumentSourceGroup::optimize() {
@@ -244,13 +256,10 @@ Value DocumentSourceGroup::serialize(boost::optional<ExplainOptions::Verbosity>
         insides["$doingMerge"] = Value(true);
     }
 
-    if (explain && findRelevantInputSort()) {
-        return Value(DOC("$streamingGroup" << insides.freeze()));
-    }
     return Value(DOC(getSourceName() << insides.freeze()));
 }
 
-DocumentSource::GetDepsReturn DocumentSourceGroup::getDependencies(DepsTracker* deps) const {
+DepsTracker::State DocumentSourceGroup::getDependencies(DepsTracker* deps) const {
     // add the _id
     for (size_t i = 0; i < _idExpressions.size(); i++) {
         _idExpressions[i]->addDependencies(deps);
@@ -261,16 +270,54 @@ DocumentSource::GetDepsReturn DocumentSourceGroup::getDependencies(DepsTracker*
         accumulatedField.expression->addDependencies(deps);
     }
 
-    return EXHAUSTIVE_ALL;
+    return DepsTracker::State::EXHAUSTIVE_ALL;
+}
+
+DocumentSource::GetModPathsReturn DocumentSourceGroup::getModifiedPaths() const {
+    // We preserve none of the fields, but any fields referenced as part of the group key are
+    // logically just renamed.
+    StringMap<std::string> renames;
+    for (std::size_t i = 0; i < _idExpressions.size(); ++i) {
+        auto idExp = _idExpressions[i];
+        auto pathToPutResultOfExpression =
+            _idFieldNames.empty() ? "_id" : "_id." + _idFieldNames[i];
+        auto computedPaths = idExp->getComputedPaths(pathToPutResultOfExpression);
+        for (auto&& rename : computedPaths.renames) {
+            renames[rename.first] = rename.second;
+        }
+    }
+
+    return {DocumentSource::GetModPathsReturn::Type::kAllExcept,
+            std::set<std::string>{},  // No fields are preserved.
+            std::move(renames)};
+}
+
+StringMap<boost::intrusive_ptr<Expression>> DocumentSourceGroup::getIdFields() const {
+    if (_idFieldNames.empty()) {
+        invariant(_idExpressions.size() == 1);
+        return {{"_id", _idExpressions[0]}};
+    } else {
+        invariant(_idFieldNames.size() == _idExpressions.size());
+        StringMap<boost::intrusive_ptr<Expression>> result;
+        for (std::size_t i = 0; i < _idFieldNames.size(); ++i) {
+            result["_id." + _idFieldNames[i]] = _idExpressions[i];
+        }
+        return result;
+    }
+}
+
+const std::vector<AccumulationStatement>& DocumentSourceGroup::getAccumulatedFields() const {
+    return _accumulatedFields;
 }
 
 intrusive_ptr<DocumentSourceGroup> DocumentSourceGroup::create(
     const intrusive_ptr<ExpressionContext>& pExpCtx,
     const boost::intrusive_ptr<Expression>& groupByExpression,
     std::vector<AccumulationStatement> accumulationStatements,
-    size_t maxMemoryUsageBytes) {
-    intrusive_ptr<DocumentSourceGroup> groupStage(
-        new DocumentSourceGroup(pExpCtx, maxMemoryUsageBytes));
+    boost::optional<size_t> maxMemoryUsageBytes) {
+    size_t memoryBytes = maxMemoryUsageBytes ? *maxMemoryUsageBytes
+                                             : internalDocumentSourceGroupMaxMemoryBytes.load();
+    intrusive_ptr<DocumentSourceGroup> groupStage(new DocumentSourceGroup(pExpCtx, memoryBytes));
     groupStage->setIdExpression(groupByExpression);
     for (auto&& statement : accumulationStatements) {
         groupStage->addAccumulator(statement);
@@ -280,12 +327,12 @@ intrusive_ptr<DocumentSourceGroup> DocumentSourceGroup::create(
 }
 
 DocumentSourceGroup::DocumentSourceGroup(const intrusive_ptr<ExpressionContext>& pExpCtx,
-                                         size_t maxMemoryUsageBytes)
+                                         boost::optional<size_t> maxMemoryUsageBytes)
     : DocumentSource(pExpCtx),
+      _usedDisk(false),
       _doingMerge(false),
-      _maxMemoryUsageBytes(maxMemoryUsageBytes),
-      _inputSort(BSONObj()),
-      _streaming(false),
+      _maxMemoryUsageBytes(maxMemoryUsageBytes ? *maxMemoryUsageBytes
+                                               : internalDocumentSourceGroupMaxMemoryBytes.load()),
       _initialized(false),
       _groups(pExpCtx->getValueComparator().makeUnorderedValueMap<Accumulators>()),
       _spilled(false),
@@ -367,14 +414,14 @@ intrusive_ptr<DocumentSource> DocumentSourceGroup::createFromBson(
     VariablesParseState vps = pExpCtx->variablesParseState;
     while (groupIterator.more()) {
         BSONElement groupField(groupIterator.next());
-        const char* pFieldName = groupField.fieldName();
+        StringData pFieldName = groupField.fieldNameStringData();
 
-        if (str::equals(pFieldName, "_id")) {
+        if (pFieldName == "_id") {
             uassert(
                 15948, "a group's _id may only be specified once", pGroup->_idExpressions.empty());
             pGroup->setIdExpression(parseIdExpression(pExpCtx, groupField, vps));
             invariant(!pGroup->_idExpressions.empty());
-        } else if (str::equals(pFieldName, "$doingMerge")) {
+        } else if (pFieldName == "$doingMerge") {
             massert(17030, "$doingMerge should be true if present", groupField.Bool());
 
             pGroup->setDoingMerge(true);
@@ -418,100 +465,11 @@ public:
 private:
     ValueComparator _valueComparator;
 };
-
-bool containsOnlyFieldPathsAndConstants(ExpressionObject* expressionObj) {
-    for (auto&& it : expressionObj->getChildExpressions()) {
-        const intrusive_ptr<Expression>& childExp = it.second;
-        if (dynamic_cast<ExpressionFieldPath*>(childExp.get())) {
-            continue;
-        } else if (dynamic_cast<ExpressionConstant*>(childExp.get())) {
-            continue;
-        } else if (auto expObj = dynamic_cast<ExpressionObject*>(childExp.get())) {
-            if (!containsOnlyFieldPathsAndConstants(expObj)) {
-                // A nested expression was not a FieldPath or a constant.
-                return false;
-            }
-        } else {
-            // expressionObj was something other than a FieldPath, a constant, or a nested object.
-            return false;
-        }
-    }
-    return true;
-}
-
-void getFieldPathMap(ExpressionObject* expressionObj,
-                     std::string prefix,
-                     StringMap<std::string>* fields) {
-    // Given an expression with only constant and FieldPath leaf nodes, such as {x: {y: "$a.b"}},
-    // attempt to compute a map from each FieldPath leaf to the path of that leaf. In the example,
-    // this method would return: {"a.b" : "x.y"}.
-
-    for (auto&& it : expressionObj->getChildExpressions()) {
-        intrusive_ptr<Expression> childExp = it.second;
-        ExpressionObject* expObj = dynamic_cast<ExpressionObject*>(childExp.get());
-        ExpressionFieldPath* expPath = dynamic_cast<ExpressionFieldPath*>(childExp.get());
-
-        std::string newPrefix = prefix.empty() ? it.first : prefix + "." + it.first;
-
-        if (expObj) {
-            getFieldPathMap(expObj, newPrefix, fields);
-        } else if (expPath) {
-            (*fields)[expPath->getFieldPath().tail().fullPath()] = newPrefix;
-        }
-    }
-}
-
-void getFieldPathListForSpilled(ExpressionObject* expressionObj,
-                                std::string prefix,
-                                std::vector<std::string>* fields) {
-    // Given an expression, attempt to compute a vector of strings, each representing the path
-    // through the object to a leaf. For example, for the expression represented by
-    // {x: 2, y: {z: "$a.b"}}, the output would be ["x", "y.z"].
-    for (auto&& it : expressionObj->getChildExpressions()) {
-        intrusive_ptr<Expression> childExp = it.second;
-        ExpressionObject* expObj = dynamic_cast<ExpressionObject*>(childExp.get());
-
-        std::string newPrefix = prefix.empty() ? it.first : prefix + "." + it.first;
-
-        if (expObj) {
-            getFieldPathListForSpilled(expObj, newPrefix, fields);
-        } else {
-            fields->push_back(newPrefix);
-        }
-    }
-}
 }  // namespace
 
 DocumentSource::GetNextResult DocumentSourceGroup::initialize() {
     const size_t numAccumulators = _accumulatedFields.size();
 
-    boost::optional<BSONObj> inputSort = findRelevantInputSort();
-    if (inputSort) {
-        // We can convert to streaming.
-        _streaming = true;
-        _inputSort = *inputSort;
-
-        // Set up accumulators.
-        _currentAccumulators.reserve(numAccumulators);
-        for (auto&& accumulatedField : _accumulatedFields) {
-            _currentAccumulators.push_back(accumulatedField.makeAccumulator(pExpCtx));
-        }
-
-        // We only need to load the first document.
-        auto firstInput = pSource->getNext();
-        if (!firstInput.isAdvanced()) {
-            // Leave '_firstDocOfNextGroup' uninitialized and return.
-            return firstInput;
-        }
-        _firstDocOfNextGroup = firstInput.releaseDocument();
-
-        // Compute the _id value.
-        _currentId = computeId(*_firstDocOfNextGroup);
-        _initialized = true;
-        return DocumentSource::GetNextResult::makeEOF();
-    }
-
-
     // Barring any pausing, this loop exhausts 'pSource' and populates '_groups'.
     GetNextResult input = pSource->getNext();
     for (; input.isAdvanced(); input = pSource->getNext()) {
@@ -555,8 +513,9 @@ DocumentSource::GetNextResult DocumentSourceGroup::initialize() {
         dassert(numAccumulators == group.size());
 
         for (size_t i = 0; i < numAccumulators; i++) {
-            group[i]->process(_accumulatedFields[i].expression->evaluate(rootDocument),
-                              _doingMerge);
+            group[i]->process(
+                _accumulatedFields[i].expression->evaluate(rootDocument, &pExpCtx->variables),
+                _doingMerge);
 
             _memoryUsageBytes += group[i]->memUsageForSorter();
         }
@@ -620,7 +579,12 @@ DocumentSource::GetNextResult DocumentSourceGroup::initialize() {
     MONGO_UNREACHABLE;
 }
 
+bool DocumentSourceGroup::usedDisk() {
+    return _usedDisk;
+}
+
 shared_ptr<Sorter<Value, Value>::Iterator> DocumentSourceGroup::spill() {
+    _usedDisk = true;
     vector<const GroupsMap::value_type*> ptrs;  // using pointers to speed sorting
     ptrs.reserve(_groups->size());
     for (GroupsMap::const_iterator it = _groups->begin(), end = _groups->end(); it != end; ++it) {
@@ -663,158 +627,10 @@ shared_ptr<Sorter<Value, Value>::Iterator> DocumentSourceGroup::spill() {
     return shared_ptr<Sorter<Value, Value>::Iterator>(iteratorPtr);
 }
 
-boost::optional<BSONObj> DocumentSourceGroup::findRelevantInputSort() const {
-    if (true) {
-        // Until streaming $group correctly handles nullish values, the streaming behavior is
-        // disabled. See SERVER-23318.
-        return boost::none;
-    }
-
-    if (!pSource) {
-        // Sometimes when performing an explain, or using $group as the merge point, 'pSource' will
-        // not be set.
-        return boost::none;
-    }
-
-    BSONObjSet sorts = pSource->getOutputSorts();
-
-    // 'sorts' is a BSONObjSet. We need to check if our group pattern is compatible with one of the
-    // input sort patterns.
-
-    // We will only attempt to take advantage of a sorted input stream if the _id given to the
-    // $group contained only FieldPaths or constants. Determine if this is the case, and extract
-    // those FieldPaths if it is.
-    DepsTracker deps(DepsTracker::MetadataAvailable::kNoMetadata);  // We don't support streaming
-                                                                    // based off a text score.
-    for (auto&& exp : _idExpressions) {
-        if (dynamic_cast<ExpressionConstant*>(exp.get())) {
-            continue;
-        }
-        ExpressionObject* obj;
-        if ((obj = dynamic_cast<ExpressionObject*>(exp.get()))) {
-            // We can only perform an optimization if there are no operators in the _id expression.
-            if (!containsOnlyFieldPathsAndConstants(obj)) {
-                return boost::none;
-            }
-        } else if (!dynamic_cast<ExpressionFieldPath*>(exp.get())) {
-            return boost::none;
-        }
-        exp->addDependencies(&deps);
-    }
-
-    if (deps.needWholeDocument) {
-        // We don't swap to streaming if we need the entire document, which is likely because of
-        // $$ROOT.
-        return boost::none;
-    }
-
-    if (deps.fields.empty()) {
-        // Our _id field is constant, so we should stream, but the input sort we choose is
-        // irrelevant since we will output only one document.
-        return BSONObj();
-    }
-
-    for (auto&& obj : sorts) {
-        // Note that a sort order of, e.g., {a: 1, b: 1, c: 1} allows us to do a non-blocking group
-        // for every permutation of group by (a, b, c), since we are guaranteed that documents with
-        // the same value of (a, b, c) will be consecutive in the input stream, no matter what our
-        // _id is.
-        std::set<std::string> fieldNames;
-        obj.getFieldNames(fieldNames);
-        if (fieldNames == deps.fields) {
-            return obj;
-        }
-    }
-
-    return boost::none;
-}
-
-BSONObjSet DocumentSourceGroup::getOutputSorts() {
-    if (!_initialized) {
-        initialize();  // Note this might not finish initializing, but that's OK. We just want to
-                       // do some initialization to try to determine if we are streaming or spilled.
-                       // False negatives are OK.
-    }
-
-    if (!(_streaming || _spilled)) {
-        return SimpleBSONObjComparator::kInstance.makeBSONObjSet();
-    }
-
-    BSONObjBuilder sortOrder;
-
-    if (_idFieldNames.empty()) {
-        if (_spilled) {
-            sortOrder.append("_id", 1);
-        } else {
-            // We have an expression like {_id: "$a"}. Check if this is a FieldPath, and if it is,
-            // get the sort order out of it.
-            if (auto obj = dynamic_cast<ExpressionFieldPath*>(_idExpressions[0].get())) {
-                FieldPath _idSort = obj->getFieldPath();
-
-                sortOrder.append(
-                    "_id",
-                    _inputSort.getIntField(_idSort.getFieldName(_idSort.getPathLength() - 1)));
-            }
-        }
-    } else if (_streaming) {
-        // At this point, we know that _streaming is true, so _id must have only contained
-        // ExpressionObjects, ExpressionConstants or ExpressionFieldPaths. We now process each
-        // '_idExpression'.
-
-        // We populate 'fieldMap' such that each key is a field the input is sorted by, and the
-        // value is where that input field is located within the _id document. For example, if our
-        // _id object is {_id: {x: {y: "$a.b"}}}, 'fieldMap' would be: {'a.b': '_id.x.y'}.
-        StringMap<std::string> fieldMap;
-        for (size_t i = 0; i < _idFieldNames.size(); i++) {
-            intrusive_ptr<Expression> exp = _idExpressions[i];
-            if (auto obj = dynamic_cast<ExpressionObject*>(exp.get())) {
-                // _id is an object containing a nested document, such as: {_id: {x: {y: "$b"}}}.
-                getFieldPathMap(obj, "_id." + _idFieldNames[i], &fieldMap);
-            } else if (auto fieldPath = dynamic_cast<ExpressionFieldPath*>(exp.get())) {
-                FieldPath _idSort = fieldPath->getFieldPath();
-                fieldMap[_idSort.getFieldName(_idSort.getPathLength() - 1)] =
-                    "_id." + _idFieldNames[i];
-            }
-        }
-
-        // Because the order of '_inputSort' is important, we go through each field we are sorted on
-        // and append it to the BSONObjBuilder in order.
-        for (BSONElement sortField : _inputSort) {
-            std::string sortString = sortField.fieldNameStringData().toString();
-
-            auto itr = fieldMap.find(sortString);
-
-            // If our sort order is (a, b, c), we could not have converted to a streaming $group if
-            // our _id was predicated on (a, c) but not 'b'. Verify that this is true.
-            invariant(itr != fieldMap.end());
-
-            sortOrder.append(itr->second, _inputSort.getIntField(sortString));
-        }
-    } else {
-        // We are blocking and have spilled to disk.
-        std::vector<std::string> outputSort;
-        for (size_t i = 0; i < _idFieldNames.size(); i++) {
-            intrusive_ptr<Expression> exp = _idExpressions[i];
-            if (auto obj = dynamic_cast<ExpressionObject*>(exp.get())) {
-                // _id is an object containing a nested document, such as: {_id: {x: {y: "$b"}}}.
-                getFieldPathListForSpilled(obj, "_id." + _idFieldNames[i], &outputSort);
-            } else {
-                outputSort.push_back("_id." + _idFieldNames[i]);
-            }
-        }
-        for (auto&& field : outputSort) {
-            sortOrder.append(field, 1);
-        }
-    }
-
-    return allPrefixes(sortOrder.obj());
-}
-
-
 Value DocumentSourceGroup::computeId(const Document& root) {
     // If only one expression, return result directly
     if (_idExpressions.size() == 1) {
-        Value retValue = _idExpressions[0]->evaluate(root);
+        Value retValue = _idExpressions[0]->evaluate(root, &pExpCtx->variables);
         return retValue.missing() ? Value(BSONNULL) : std::move(retValue);
     }
 
@@ -822,7 +638,7 @@ Value DocumentSourceGroup::computeId(const Document& root) {
     vector<Value> vals;
     vals.reserve(_idExpressions.size());
     for (size_t i = 0; i < _idExpressions.size(); i++) {
-        vals.push_back(_idExpressions[i]->evaluate(root));
+        vals.push_back(_idExpressions[i]->evaluate(root, &pExpCtx->variables));
     }
     return Value(std::move(vals));
 }
@@ -869,17 +685,13 @@ Document DocumentSourceGroup::makeDocument(const Value& id,
     return out.freeze();
 }
 
-intrusive_ptr<DocumentSource> DocumentSourceGroup::getShardSource() {
-    return this;  // No modifications necessary when on shard
-}
-
-std::list<intrusive_ptr<DocumentSource>> DocumentSourceGroup::getMergeSources() {
-    intrusive_ptr<DocumentSourceGroup> pMerger(new DocumentSourceGroup(pExpCtx));
-    pMerger->setDoingMerge(true);
+boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceGroup::distributedPlanLogic() {
+    intrusive_ptr<DocumentSourceGroup> mergingGroup(new DocumentSourceGroup(pExpCtx));
+    mergingGroup->setDoingMerge(true);
 
     VariablesParseState vps = pExpCtx->variablesParseState;
     /* the merger will use the same grouping key */
-    pMerger->setIdExpression(ExpressionFieldPath::parse(pExpCtx, "$$ROOT._id", vps));
+    mergingGroup->setIdExpression(ExpressionFieldPath::parse(pExpCtx, "$$ROOT._id", vps));
 
     for (auto&& accumulatedField : _accumulatedFields) {
         // The merger's output field names will be the same, as will the accumulator factories.
@@ -889,10 +701,90 @@ std::list<intrusive_ptr<DocumentSource>> DocumentSourceGroup::getMergeSources()
         auto copiedAccumuledField = accumulatedField;
         copiedAccumuledField.expression =
             ExpressionFieldPath::parse(pExpCtx, "$$ROOT." + accumulatedField.fieldName, vps);
-        pMerger->addAccumulator(copiedAccumuledField);
+        mergingGroup->addAccumulator(copiedAccumuledField);
     }
 
-    return {pMerger};
+    // {shardsStage, mergingStage, sortPattern}
+    return DistributedPlanLogic{this, mergingGroup, boost::none};
+}
+
+bool DocumentSourceGroup::pathIncludedInGroupKeys(const std::string& dottedPath) const {
+    return std::any_of(
+        _idExpressions.begin(), _idExpressions.end(), [&dottedPath](const auto& exp) {
+            if (auto fieldExp = dynamic_cast<ExpressionFieldPath*>(exp.get())) {
+                if (fieldExp->representsPath(dottedPath)) {
+                    return true;
+                }
+            }
+            return false;
+        });
+}
+
+bool DocumentSourceGroup::canRunInParallelBeforeWriteStage(
+    const std::set<std::string>& nameOfShardKeyFieldsUponEntryToStage) const {
+    if (_doingMerge) {
+        return true;  // This is fine.
+    }
+
+    // Certain $group stages are allowed to execute on each exchange consumer. In order to
+    // guarantee each consumer will only group together data from its own shard, the $group must
+    // group on a superset of the shard key.
+    for (auto&& currentPathOfShardKey : nameOfShardKeyFieldsUponEntryToStage) {
+        if (!pathIncludedInGroupKeys(currentPathOfShardKey)) {
+            // This requires an exact path match, but as a future optimization certain path
+            // prefixes should be okay. For example, if the shard key path is "a.b", and we're
+            // grouping by "a", then each group of "a" is strictly more specific than "a.b", so
+            // we can deduce that grouping by "a" will not need to group together documents
+            // across different values of the shard key field "a.b", and thus as long as any
+            // other shard key fields are similarly preserved will not need to consume a merged
+            // stream to perform the group.
+            return false;
+        }
+    }
+    return true;
+}
+
+std::unique_ptr<GroupFromFirstDocumentTransformation>
+DocumentSourceGroup::rewriteGroupAsTransformOnFirstDocument() const {
+    if (!_idFieldNames.empty()) {
+        // This transformation is only intended for $group stages that group on a single field.
+        return nullptr;
+    }
+
+    invariant(_idExpressions.size() == 1);
+    auto fieldPathExpr = dynamic_cast<ExpressionFieldPath*>(_idExpressions.front().get());
+    if (!fieldPathExpr || !fieldPathExpr->isRootFieldPath()) {
+        return nullptr;
+    }
+
+    const auto fieldPath = fieldPathExpr->getFieldPath();
+    if (fieldPath.getPathLength() == 1) {
+        // The path is $$CURRENT or $$ROOT. This isn't really a sensible value to group by (since
+        // each document has a unique _id, it will just return the entire collection). We only
+        // apply the rewrite when grouping by a single field, so we cannot apply it in this case,
+        // where we are grouping by the entire document.
+        invariant(fieldPath.getFieldName(0) == "CURRENT" || fieldPath.getFieldName(0) == "ROOT");
+        return nullptr;
+    }
+
+    const auto groupId = fieldPath.tail().fullPath();
+
+    // We can't do this transformation if there are any non-$first accumulators.
+    for (auto&& accumulator : _accumulatedFields) {
+        if (AccumulatorDocumentsNeeded::kFirstDocument !=
+            accumulator.makeAccumulator(pExpCtx)->documentsNeeded()) {
+            return nullptr;
+        }
+    }
+
+    std::vector<std::pair<std::string, boost::intrusive_ptr<Expression>>> fields;
+    fields.push_back(std::make_pair("_id", ExpressionFieldPath::create(pExpCtx, groupId)));
+
+    for (auto&& accumulator : _accumulatedFields) {
+        fields.push_back(std::make_pair(accumulator.fieldName, accumulator.expression));
+    }
+
+    return GroupFromFirstDocumentTransformation::create(pExpCtx, groupId, std::move(fields));
 }
 
 }  // namespace mongo
