@@ -141,6 +141,11 @@ public:
         const boost::optional<ExplainOptions::Verbosity> explain,
         bool enableMajorityReadConcern) const;
 
+    /**
+     * Increments global stage counters corresponding to the stages in this lite parsed pipeline.
+     */
+    void tickGlobalStageCounters() const;
+
 private:
     std::vector<std::unique_ptr<LiteParsedDocumentSource>> _stageSpecs;
 };
