@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -40,7 +39,7 @@
 #include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/value.h"
 #include "mongo/db/query/explain_options.h"
-#include "mongo/db/query/query_knobs.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/s/query/async_results_merger_params_gen.h"
 #include "mongo/stdx/functional.h"
@@ -50,9 +49,9 @@
 namespace mongo {
 class BSONObj;
 class BSONObjBuilder;
-class ExpressionContext;
-class DocumentSource;
 class CollatorInterface;
+class DocumentSource;
+class ExpressionContext;
 class OperationContext;
 class PipelineDeleter;
 
@@ -71,13 +70,22 @@ public:
     enum class SplitState { kUnsplit, kSplitForShards, kSplitForMerge };
 
     /**
-     * List of supported match expression features in a pipeline.
+     * The list of default supported match expression features.
      */
     static constexpr MatchExpressionParser::AllowedFeatureSet kAllowedMatcherFeatures =
         MatchExpressionParser::AllowedFeatures::kText |
         MatchExpressionParser::AllowedFeatures::kExpr |
         MatchExpressionParser::AllowedFeatures::kJSONSchema;
 
+    /**
+     * The match expression features allowed when running a pipeline with $geoNear.
+     */
+    static constexpr MatchExpressionParser::AllowedFeatureSet kGeoNearMatcherFeatures =
+        MatchExpressionParser::AllowedFeatures::kText |
+        MatchExpressionParser::AllowedFeatures::kExpr |
+        MatchExpressionParser::AllowedFeatures::kJSONSchema |
+        MatchExpressionParser::AllowedFeatures::kGeoNear;
+
     /**
      * Parses a Pipeline from a vector of BSONObjs. Returns a non-OK status if it failed to parse.
      * The returned pipeline is not optimized, but the caller may convert it to an optimized
@@ -121,9 +129,22 @@ public:
         SourceContainer sources, const boost::intrusive_ptr<ExpressionContext>& expCtx);
 
     /**
-     * Returns true if the provided aggregation command has a $out stage.
+     * Returns true if the provided aggregation command has an $out or $merge stage.
      */
-    static bool aggSupportsWriteConcern(const BSONObj& cmd);
+    static bool aggHasWriteStage(const BSONObj& cmd);
+
+    /**
+     * Given 'pathsOfInterest' which describes a set of paths which the caller is interested in,
+     * returns boost::none if any of those paths are modified by the section of a pipeline
+     * described by 'rstart' and 'rend', or a mapping from their name at the end of the pipeline to
+     * their name at the beginning of the pipeline if they are preserved but possibly renamed by
+     * this pipeline. Note that the analysis proceeds backwards, so the iterators must be reverse
+     * iterators.
+     */
+    static boost::optional<StringMap<std::string>> renamedPaths(
+        SourceContainer::const_reverse_iterator rstart,
+        SourceContainer::const_reverse_iterator rend,
+        std::set<std::string> pathsOfInterest);
 
     const boost::intrusive_ptr<ExpressionContext>& getContext() const {
         return pCtx;
@@ -160,37 +181,21 @@ public:
     void dispose(OperationContext* opCtx);
 
     /**
-     * Split the current Pipeline into a Pipeline for each shard, and a Pipeline that combines the
-     * results within mongos. This permanently alters this pipeline for the merging operation, and
-     * returns a Pipeline object that should be executed on each targeted shard.
-    */
-    std::unique_ptr<Pipeline, PipelineDeleter> splitForSharded();
+     * Checks to see if disk is ever used within the pipeline.
+     */
+    bool usedDisk();
 
     /**
-     * Returns true if this pipeline has not been split.
+     * Communicates to the pipeline which part of a split pipeline it is when the pipeline has been
+     * split in two.
      */
-    bool isUnsplit() const {
-        return _splitState == SplitState::kUnsplit;
+    void setSplitState(SplitState state) {
+        _splitState = state;
     }
 
     /**
-     * Returns true if this pipeline is the part of a split pipeline which should be targeted to the
-     * shards.
-     */
-    bool isSplitForShards() const {
-        return _splitState == SplitState::kSplitForShards;
-    }
-
-    /**
-     * Returns true if this pipeline is the part of a split pipeline which is responsible for
-     * merging the results from the shards.
-     */
-    bool isSplitForMerge() const {
-        return _splitState == SplitState::kSplitForMerge;
-    }
-
-    /** If the pipeline starts with a $match, return its BSON predicate.
-     *  Returns empty BSON if the first stage isn't $match.
+     * If the pipeline starts with a stage which is or includes a query predicate (e.g. a $match),
+     * returns a BSON object representing that query. Otherwise, returns an empty BSON object.
      */
     BSONObj getInitialQuery() const;
 
@@ -228,9 +233,10 @@ public:
 
     /**
      * Returns any other collections involved in the pipeline in addition to the collection the
-     * aggregation is run on.
+     * aggregation is run on. All namespaces returned are the names of collections, after views have
+     * been resolved.
      */
-    std::vector<NamespaceString> getInvolvedCollections() const;
+    stdx::unordered_set<NamespaceString> getInvolvedCollections() const;
 
     /**
      * Serializes the pipeline into a form that can be parsed into an equivalent pipeline.
@@ -259,10 +265,23 @@ public:
      */
     DepsTracker getDependencies(DepsTracker::MetadataAvailable metadataAvailable) const;
 
+    /**
+     * Given 'pathsOfInterest' which describes a set of paths which the caller is interested in,
+     * returns boost::none if any of those paths are modified by this pipeline, or a mapping from
+     * their name at the end of the pipeline to their name at the beginning of the pipeline if they
+     * are preserved but possibly renamed by this pipeline.
+     */
+    boost::optional<StringMap<std::string>> renamedPaths(
+        std::set<std::string> pathsOfInterest) const;
+
     const SourceContainer& getSources() const {
         return _sources;
     }
 
+    SourceContainer& getSources() {
+        return _sources;
+    }
+
     /**
      * Removes and returns the first stage of the pipeline. Returns nullptr if the pipeline is
      * empty.
