@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -37,7 +36,7 @@
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/util/mongoutils/str.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace {
@@ -50,8 +49,7 @@ ServiceContext::ConstructorActionRegisterer loadTimeZoneDB{
             if (!timeZoneDatabase) {
                 uasserted(ErrorCodes::FailedToParse,
                           str::stream() << "failed to load time zone database from path \""
-                                        << serverGlobalParams.timeZoneInfoPath
-                                        << "\"");
+                                        << serverGlobalParams.timeZoneInfoPath << "\"");
             }
             TimeZoneDatabase::set(service,
                                   stdx::make_unique<TimeZoneDatabase>(std::move(timeZoneDatabase)));
