@@ -1,23 +1,24 @@
 /**
- *    Copyright (C) 2013 10gen Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -29,11 +30,9 @@
 #pragma once
 
 #include <boost/optional.hpp>
-#include <queue>
 
 #include "mongo/base/status.h"
 #include "mongo/db/catalog/util/partitioned.h"
-#include "mongo/db/invalidation_type.h"
 #include "mongo/db/query/query_solution.h"
 #include "mongo/db/storage/snapshot.h"
 #include "mongo/stdx/unordered_set.h"
@@ -44,7 +43,6 @@ class BSONObj;
 class CappedInsertNotifier;
 struct CappedInsertNotifierData;
 class Collection;
-class CursorManager;
 class PlanExecutor;
 class PlanStage;
 class PlanYieldPolicy;
@@ -79,18 +77,22 @@ public:
         // We're EOF.  We won't return any more results (edge case exception: capped+tailable).
         IS_EOF,
 
-        // We were killed. This is a special failure case in which we cannot rely on the
-        // collection or database to still be valid.
-        // If the underlying PlanStage has any information on the error, it will be available in
-        // the objOut parameter. Call WorkingSetCommon::toStatusString() to retrieve the error
-        // details from the output BSON object.
-        DEAD,
-
-        // getNext was asked for data it cannot provide, or the underlying PlanStage had an
-        // unrecoverable error.
+        // getNext() was asked for data it cannot provide, or the underlying PlanStage had an
+        // unrecoverable error, or the executor died, usually due to a concurrent catalog event
+        // such as a collection drop.
+        //
         // If the underlying PlanStage has any information on the error, it will be available in
         // the objOut parameter. Call WorkingSetCommon::toStatusString() to retrieve the error
         // details from the output BSON object.
+        //
+        // The PlanExecutor is no longer capable of executing. The caller may extract stats from the
+        // underlying plan stages, but should not attempt to do anything else with the executor
+        // other than dispose() and destroy it.
+        //
+        // N.B.: If the plan's YieldPolicy allows yielding, FAILURE can be returned on interrupt,
+        // and any locks acquired might possibly be released, regardless of the use of any RAII
+        // locking helpers such as AutoGetCollection.  Code must be written to expect this
+        // situation.
         FAILURE,
     };
 
@@ -99,20 +101,28 @@ public:
      * (NO_YIELD).
      */
     enum YieldPolicy {
-        // Any call to getNext() may yield. In particular, the executor may be killed during any
-        // call to getNext().  If this occurs, getNext() will return DEAD. Additionally, this
-        // will handle all WriteConflictExceptions that occur while processing the query.
+        // Any call to getNext() may yield. In particular, the executor may die on any call to
+        // getNext() due to a required index or collection becoming invalid during yield. If this
+        // occurs, getNext() will produce an error during yield recovery and will return FAILURE.
+        // Additionally, this will handle all WriteConflictExceptions that occur while processing
+        // the query.  With this yield policy, it is possible for getNext() to return FAILURE with
+        // locks released, if the operation is killed while yielding.
         YIELD_AUTO,
 
-        // This will handle WriteConflictExceptions that occur while processing the query, but
-        // will not yield locks. abandonSnapshot() will be called if a WriteConflictException
-        // occurs so callers must be prepared to get a new snapshot. A PlanExecutor constructed with
-        // this yield policy will not be registered to receive invalidations, so the caller must
-        // hold their locks continuously from construction to destruction.
+        // This will handle WriteConflictExceptions that occur while processing the query, but will
+        // not yield locks. abandonSnapshot() will be called if a WriteConflictException occurs so
+        // callers must be prepared to get a new snapshot. The caller must hold their locks
+        // continuously from construction to destruction. Callers which do not want auto-yielding,
+        // but may release their locks during query execution must use the YIELD_MANUAL policy.
         WRITE_CONFLICT_RETRY_ONLY,
 
         // Use this policy if you want to disable auto-yielding, but will release locks while using
         // the PlanExecutor. Any WriteConflictExceptions will be raised to the caller of getNext().
+        //
+        // With this policy, an explicit call must be made to saveState() before releasing locks,
+        // and an explicit call to restoreState() must be made after reacquiring locks.
+        // restoreState() will throw if the PlanExecutor is now invalid due to a catalog operation
+        // (e.g. collection drop) during yield.
         YIELD_MANUAL,
 
         // Can be used in one of the following scenarios:
@@ -126,12 +136,12 @@ public:
         INTERRUPT_ONLY,
 
         // Used for testing, this yield policy will cause the PlanExecutor to time out on the first
-        // yield, returning DEAD with an error object encoding a ErrorCodes::ExceededTimeLimit
+        // yield, returning FAILURE with an error object encoding a ErrorCodes::ExceededTimeLimit
         // message.
         ALWAYS_TIME_OUT,
 
         // Used for testing, this yield policy will cause the PlanExecutor to be marked as killed on
-        // the first yield, returning DEAD with an error object encoding a
+        // the first yield, returning FAILURE with an error object encoding a
         // ErrorCodes::QueryPlanKilled message.
         ALWAYS_MARK_KILLED,
     };
@@ -145,9 +155,9 @@ public:
          * Constructs an empty deleter. Useful for creating a
          * unique_ptr<PlanExecutor, PlanExecutor::Deleter> without populating it.
          */
-        Deleter() {}
+        Deleter() = default;
 
-        Deleter(OperationContext* opCtx, const Collection* collection);
+        inline Deleter(OperationContext* opCtx) : _opCtx(opCtx) {}
 
         /**
          * If an owner of a std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> wants to assume
@@ -159,15 +169,25 @@ public:
         }
 
         /**
-         * If 'execPtr' hasn't already been disposed, will call dispose(). Also, if 'execPtr' has
-         * been registered with the CursorManager, will deregister it. If 'execPtr' is a yielding
-         * PlanExecutor, callers must hold a lock on the collection in at least MODE_IS.
+         * If 'execPtr' hasn't already been disposed, will call dispose(). If 'execPtr' is a
+         * yielding PlanExecutor, callers must hold a lock on the collection in at least MODE_IS.
          */
-        void operator()(PlanExecutor* execPtr);
+        inline void operator()(PlanExecutor* execPtr) {
+            try {
+                // It is illegal to invoke operator() on a default constructed Deleter.
+                invariant(_opCtx);
+                if (!_dismissed) {
+                    execPtr->dispose(_opCtx);
+                }
+                delete execPtr;
+            } catch (...) {
+                std::terminate();
+            }
+        }
+
 
     private:
         OperationContext* _opCtx = nullptr;
-        CursorManager* _cursorManager = nullptr;
 
         bool _dismissed = false;
     };
@@ -184,8 +204,8 @@ public:
     //   - On any call to restoreState().
     //   - While executing the plan inside executePlan().
     //
-    // The executor will also be automatically registered to receive notifications in the case of
-    // YIELD_AUTO or YIELD_MANUAL.
+    // If auto-yielding is enabled, a yield during make() may result in the PlanExecutor being
+    // killed, in which case this method will return a non-OK status.
     //
 
     /**
@@ -237,6 +257,13 @@ public:
         const Collection* collection,
         YieldPolicy yieldPolicy);
 
+    /**
+     * A PlanExecutor must be disposed before destruction. In most cases, this will happen
+     * automatically through a PlanExecutor::Deleter or a ClientCursor.
+     */
+    PlanExecutor() = default;
+    virtual ~PlanExecutor() = default;
+
     //
     // Accessors
     //
@@ -244,38 +271,27 @@ public:
     /**
      * Get the working set used by this executor, without transferring ownership.
      */
-    WorkingSet* getWorkingSet() const;
+    virtual WorkingSet* getWorkingSet() const = 0;
 
     /**
      * Get the stage tree wrapped by this executor, without transferring ownership.
      */
-    PlanStage* getRootStage() const;
+    virtual PlanStage* getRootStage() const = 0;
 
     /**
      * Get the query that this executor is executing, without transferring ownership.
      */
-    CanonicalQuery* getCanonicalQuery() const;
+    virtual CanonicalQuery* getCanonicalQuery() const = 0;
 
     /**
      * Return the NS that the query is running over.
      */
-    const NamespaceString& nss() const {
-        return _nss;
-    }
+    virtual const NamespaceString& nss() const = 0;
 
     /**
      * Return the OperationContext that the plan is currently executing within.
      */
-    OperationContext* getOpCtx() const;
-
-    /**
-     * Generates a tree of stats objects with a separate lifetime from the execution
-     * stage tree wrapped by this PlanExecutor.
-     *
-     * This may be called without holding any locks. It also may be called on a PlanExecutor that
-     * has been killed or has produced an error.
-     */
-    std::unique_ptr<PlanStageStats> getStats() const;
+    virtual OperationContext* getOpCtx() const = 0;
 
     //
     // Methods that just pass down to the PlanStage tree.
@@ -287,23 +303,21 @@ public:
      * While in the "saved" state, it is only legal to call restoreState,
      * detachFromOperationContext, or the destructor.
      */
-    void saveState();
+    virtual void saveState() = 0;
 
     /**
-     * Restores the state saved by a saveState() call.
+     * Restores the state saved by a saveState() call. When this method returns successfully, the
+     * execution tree can once again be executed via work().
      *
-     * Returns Status::OK() if the state was successfully restored and the execution tree can be
-     * work()'d.
+     * Throws a UserException if the state cannot be successfully restored (e.g. a collection was
+     * dropped or the position of a capped cursor was lost during a yield). If restore fails, it is
+     * only safe to call dispose(), detachFromOperationContext(), or the destructor.
      *
-     * Returns ErrorCodes::QueryPlanKilled if the PlanExecutor was killed while saved.
-     *
-     * If allowed, will yield and retry if a WriteConflictException is encountered. If the time
-     * limit is exceeded during this retry process, returns ErrorCodes::MaxTimeMSExpired. If this
-     * PlanExecutor is killed during this retry process, returns ErrorCodes::QueryPlanKilled. In
-     * this scenario, locks will have been released, and will not be held when control returns to
-     * the caller.
+     * If allowed by the executor's yield policy, will yield and retry internally if a
+     * WriteConflictException is encountered. If the time limit is exceeded during this retry
+     * process, throws ErrorCodes::MaxTimeMSExpired.
      */
-    Status restoreState();
+    virtual void restoreState() = 0;
 
     /**
      * Detaches from the OperationContext and releases any storage-engine state.
@@ -312,7 +326,7 @@ public:
      * only legal to call reattachToOperationContext or the destructor. It is not legal to call
      * detachFromOperationContext() while already in the detached state.
      */
-    void detachFromOperationContext();
+    virtual void detachFromOperationContext() = 0;
 
     /**
      * Reattaches to the OperationContext and reacquires any storage-engine state.
@@ -320,7 +334,7 @@ public:
      * It is only legal to call this in the "detached" state. On return, the cursor is left in a
      * "saved" state, so callers must still call restoreState to use this object.
      */
-    void reattachToOperationContext(OperationContext* opCtx);
+    virtual void reattachToOperationContext(OperationContext* opCtx) = 0;
 
     /**
      * Same as restoreState but without the logic to retry if a WriteConflictException is
@@ -328,7 +342,7 @@ public:
      *
      * This is only public for PlanYieldPolicy. DO NOT CALL ANYWHERE ELSE.
      */
-    Status restoreStateWithoutRetrying();
+    virtual void restoreStateWithoutRetrying() = 0;
 
     //
     // Running Support
@@ -343,9 +357,9 @@ public:
      *
      * If a YIELD_AUTO policy is set, then this method may yield.
      */
-    ExecState getNextSnapshotted(Snapshotted<BSONObj>* objOut, RecordId* dlOut);
+    virtual ExecState getNextSnapshotted(Snapshotted<BSONObj>* objOut, RecordId* dlOut) = 0;
 
-    ExecState getNext(BSONObj* objOut, RecordId* dlOut);
+    virtual ExecState getNext(BSONObj* objOut, RecordId* dlOut) = 0;
 
     /**
      * Returns 'true' if the plan is done producing results (or writing), 'false' otherwise.
@@ -353,7 +367,7 @@ public:
      * Tailable cursors are a possible exception to this: they may have further results even if
      * isEOF() returns true.
      */
-    bool isEOF();
+    virtual bool isEOF() = 0;
 
     /**
      * Execute the plan to completion, throwing out the results.  Used when you want to work the
@@ -365,26 +379,24 @@ public:
      * error occurs, it is illegal to subsequently access the collection, since it may have been
      * dropped.
      */
-    Status executePlan();
+    virtual Status executePlan() = 0;
 
     //
     // Concurrency-related methods.
     //
 
     /**
-     * If we're yielding locks, the database we're operating over or any collection we're relying on
-     * may be dropped. Plan executors are notified of such events by calling markAsKilled().
-     * Callers must specify the reason for why this executor is being killed. Subsequent calls to
-     * getNext() will return DEAD, and fill 'objOut' with an error reflecting 'killStatus'. If this
-     * method is called multiple times, only the first 'killStatus' will be retained. It is an error
-     * to call this method with Status::OK.
+     * Notifies a PlanExecutor that it should die. Callers must specify the reason for why this
+     * executor is being killed. Subsequent calls to getNext() will return FAILURE, and fill
+     * 'objOut'
+     * with an error reflecting 'killStatus'. If this method is called multiple times, only the
+     * first 'killStatus' will be retained. It is an error to call this method with Status::OK.
      */
-    void markAsKilled(Status killStatus);
+    virtual void markAsKilled(Status killStatus) = 0;
 
     /**
      * Cleans up any state associated with this PlanExecutor. Must be called before deleting this
-     * PlanExecutor. It is illegal to use a PlanExecutor after calling dispose(). 'cursorManager'
-     * may be null.
+     * PlanExecutor. It is illegal to use a PlanExecutor after calling dispose().
      *
      * There are multiple cleanup scenarios:
      *  - This PlanExecutor will only ever use one OperationContext. In this case the
@@ -394,14 +406,7 @@ public:
      *    is the owner's responsibility to call dispose() with a valid OperationContext before
      *    deleting the PlanExecutor.
      */
-    void dispose(OperationContext* opCtx, CursorManager* cursorManager);
-
-    /**
-     * If we're yielding locks, writes may occur to documents that we rely on to keep valid
-     * state.  As such, if the plan yields, it must be notified of relevant writes so that
-     * we can ensure that it doesn't crash if we try to access invalid state.
-     */
-    void invalidate(OperationContext* opCtx, const RecordId& dl, InvalidationType type);
+    virtual void dispose(OperationContext* opCtx) = 0;
 
     /**
      * Helper method to aid in displaying an ExecState for debug or other recreational purposes.
@@ -420,169 +425,30 @@ public:
      * If used in combination with getNextSnapshotted(), then the SnapshotId associated with
      * 'obj' will be null when 'obj' is dequeued.
      */
-    void enqueue(const BSONObj& obj);
+    virtual void enqueue(const BSONObj& obj) = 0;
 
-    /**
-     * Helper method which returns a set of BSONObj, where each represents a sort order of our
-     * output.
-     */
-    BSONObjSet getOutputSorts() const;
+    virtual bool isMarkedAsKilled() const = 0;
+    virtual Status getKillStatus() = 0;
 
-    /**
-     * Communicate to this PlanExecutor that it is no longer registered with the CursorManager as a
-     * 'non-cached PlanExecutor'.
-     */
-    void unsetRegistered() {
-        _registrationToken.reset();
-    }
-
-    boost::optional<Partitioned<stdx::unordered_set<PlanExecutor*>>::PartitionId>
-    getRegistrationToken() const& {
-        return _registrationToken;
-    }
-    void getRegistrationToken() && = delete;
-
-    void setRegistrationToken(
-        Partitioned<stdx::unordered_set<PlanExecutor*>>::PartitionId token) & {
-        invariant(!_registrationToken);
-        _registrationToken = token;
-    }
-
-    bool isMarkedAsKilled() const {
-        return !_killStatus.isOK();
-    }
-
-    Status getKillStatus() {
-        invariant(isMarkedAsKilled());
-        return _killStatus;
-    }
-
-    bool isDisposed() const {
-        return _currentState == kDisposed;
-    }
+    virtual bool isDisposed() const = 0;
+    virtual bool isDetached() const = 0;
 
     /**
      * If the last oplog timestamp is being tracked for this PlanExecutor, return it.
      * Otherwise return a null timestamp.
      */
-    Timestamp getLatestOplogTimestamp();
-
-private:
-    /**
-     * Returns true if the PlanExecutor should listen for inserts, which is when a getMore is called
-     * on a tailable and awaitData cursor that still has time left and hasn't been interrupted.
-     */
-    bool shouldListenForInserts();
+    virtual Timestamp getLatestOplogTimestamp() const = 0;
 
     /**
-     * Returns true if the PlanExecutor should wait for data to be inserted, which is when a getMore
-     * is called on a tailable and awaitData cursor on a capped collection.  Returns false if an EOF
-     * should be returned immediately.
+     * If this PlanExecutor is tracking change stream resume tokens, return the most recent token
+     * for the batch that is currently being built. Otherwise, return an empty object.
      */
-    bool shouldWaitForInserts();
+    virtual BSONObj getPostBatchResumeToken() const = 0;
 
     /**
-     * Gets the CappedInsertNotifier for a capped collection.  Returns nullptr if this plan executor
-     * is not capable of yielding based on a notifier.
+     * Turns a BSONObj representing an error status produced by getNext() into a Status.
      */
-    std::shared_ptr<CappedInsertNotifier> getCappedInsertNotifier();
-
-    /**
-     * Yields locks and waits for inserts to the collection. Returns ADVANCED if there has been an
-     * insertion and there may be new results. Returns DEAD if the PlanExecutor was killed during a
-     * yield. This method is only to be used for tailable and awaitData cursors, so rather than
-     * returning DEAD if the operation has exceeded its time limit, we return IS_EOF to preserve
-     * this PlanExecutor for future use.
-     *
-     * If an error is encountered and 'errorObj' is provided, it is populated with an object
-     * describing the error.
-     */
-    ExecState waitForInserts(CappedInsertNotifierData* notifierData,
-                             Snapshotted<BSONObj>* errorObj);
-
-    ExecState getNextImpl(Snapshotted<BSONObj>* objOut, RecordId* dlOut);
-
-    /**
-     * New PlanExecutor instances are created with the static make() methods above.
-     */
-    PlanExecutor(OperationContext* opCtx,
-                 std::unique_ptr<WorkingSet> ws,
-                 std::unique_ptr<PlanStage> rt,
-                 std::unique_ptr<QuerySolution> qs,
-                 std::unique_ptr<CanonicalQuery> cq,
-                 const Collection* collection,
-                 NamespaceString nss,
-                 YieldPolicy yieldPolicy);
-
-    /**
-     * A PlanExecutor must be disposed before destruction. In most cases, this will happen
-     * automatically through a PlanExecutor::Deleter or a ClientCursor.
-     */
-    ~PlanExecutor();
-
-    /**
-     * Public factory methods delegate to this private factory to do their work.
-     */
-    static StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> make(
-        OperationContext* opCtx,
-        std::unique_ptr<WorkingSet> ws,
-        std::unique_ptr<PlanStage> rt,
-        std::unique_ptr<QuerySolution> qs,
-        std::unique_ptr<CanonicalQuery> cq,
-        const Collection* collection,
-        NamespaceString nss,
-        YieldPolicy yieldPolicy);
-
-    /**
-     * Clients of PlanExecutor expect that on receiving a new instance from one of the make()
-     * factory methods, plan selection has already been completed. In order to enforce this
-     * property, this function is called to do plan selection prior to returning the new
-     * PlanExecutor.
-     *
-     * If the tree contains plan selection stages, such as MultiPlanStage or SubplanStage,
-     * this calls into their underlying plan selection facilities. Otherwise, does nothing.
-     *
-     * If a YIELD_AUTO policy is set then locks are yielded during plan selection.
-     *
-     * Returns a non-OK status if query planning fails. In particular, this function returns
-     * ErrorCodes::QueryPlanKilled if plan execution cannot proceed due to a concurrent write or
-     * catalog operation.
-     */
-    Status pickBestPlan(const Collection* collection);
-
-    // The OperationContext that we're executing within. This can be updated if necessary by using
-    // detachFromOperationContext() and reattachToOperationContext().
-    OperationContext* _opCtx;
-
-    std::unique_ptr<CanonicalQuery> _cq;
-    std::unique_ptr<WorkingSet> _workingSet;
-    std::unique_ptr<QuerySolution> _qs;
-    std::unique_ptr<PlanStage> _root;
-
-    // If _killStatus has a non-OK value, then we have been killed and the value represents the
-    // reason for the kill.
-    Status _killStatus = Status::OK();
-
-    // What namespace are we operating over?
-    NamespaceString _nss;
-
-    // This is used to handle automatic yielding when allowed by the YieldPolicy. Never NULL.
-    // TODO make this a non-pointer member. This requires some header shuffling so that this
-    // file includes plan_yield_policy.h rather than the other way around.
-    const std::unique_ptr<PlanYieldPolicy> _yieldPolicy;
-
-    // A stash of results generated by this plan that the user of the PlanExecutor didn't want
-    // to consume yet. We empty the queue before retrieving further results from the plan
-    // stages.
-    std::queue<BSONObj> _stash;
-
-    enum { kUsable, kSaved, kDetached, kDisposed } _currentState = kUsable;
-
-    // Set if this PlanExecutor is registered with the CursorManager.
-    boost::optional<Partitioned<stdx::unordered_set<PlanExecutor*>>::PartitionId>
-        _registrationToken;
-
-    bool _everDetachedFromOperationContext = false;
+    virtual Status getMemberObjectStatus(const BSONObj& memberObj) const = 0;
 };
 
 }  // namespace mongo
