@@ -207,14 +207,9 @@ double PlanRanker::scoreTree(const PlanStageStats* stats) {
     // plan doesn't lose to a less productive plan due to tie breaking.
     const double epsilon = std::min(1.0 / static_cast<double>(10 * workUnits), 1e-4);
 
-    // We prefer covered projections.
-    //
-    // We only do this when we have a projection stage because we have so many jstests that
-    // check bounds even when a collscan plan is just as good as the ixscan'd plan :(
+    // We prefer queries that don't require a fetch stage.
     double noFetchBonus = epsilon;
-    if ((hasStage(STAGE_PROJECTION_DEFAULT, stats) || hasStage(STAGE_PROJECTION_COVERED, stats) ||
-         hasStage(STAGE_PROJECTION_SIMPLE, stats)) &&
-        hasStage(STAGE_FETCH, stats)) {
+    if (hasStage(STAGE_FETCH, stats)) {
         noFetchBonus = 0;
     }
 
