@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -142,17 +141,13 @@ TEST_F(ApplyOpsTest, CommandInNestedApplyOpsReturnsSuccess) {
     auto mode = OplogApplication::Mode::kApplyOpsCmd;
     BSONObjBuilder resultBuilder;
     NamespaceString nss("test", "foo");
-    auto innerCmdObj = BSON("op"
-                            << "c"
-                            << "ns"
-                            << nss.getCommandNS().ns()
-                            << "o"
-                            << BSON("create" << nss.coll()));
+    auto innerCmdObj =
+        BSON("op"
+             << "c"
+             << "ns" << nss.getCommandNS().ns() << "o" << BSON("create" << nss.coll()));
     auto innerApplyOpsObj = BSON("op"
                                  << "c"
-                                 << "ns"
-                                 << nss.getCommandNS().ns()
-                                 << "o"
+                                 << "ns" << nss.getCommandNS().ns() << "o"
                                  << BSON("applyOps" << BSON_ARRAY(innerCmdObj)));
     auto cmdObj = BSON("applyOps" << BSON_ARRAY(innerApplyOpsObj));
 
@@ -170,18 +165,13 @@ TEST_F(ApplyOpsTest, InsertInNestedApplyOpsReturnsSuccess) {
     NamespaceString nss("test", "foo");
     auto innerCmdObj = BSON("op"
                             << "i"
-                            << "ns"
-                            << nss.ns()
-                            << "o"
+                            << "ns" << nss.ns() << "o"
                             << BSON("_id"
                                     << "a")
-                            << "ui"
-                            << options.uuid.get());
+                            << "ui" << options.uuid.get());
     auto innerApplyOpsObj = BSON("op"
                                  << "c"
-                                 << "ns"
-                                 << nss.getCommandNS().ns()
-                                 << "o"
+                                 << "ns" << nss.getCommandNS().ns() << "o"
                                  << BSON("applyOps" << BSON_ARRAY(innerCmdObj)));
     auto cmdObj = BSON("applyOps" << BSON_ARRAY(innerApplyOpsObj));
 
@@ -207,18 +197,10 @@ BSONObj makeApplyOpsWithInsertOperation(const NamespaceString& nss,
                                         const BSONObj& documentToInsert) {
     auto insertOp = uuid ? BSON("op"
                                 << "i"
-                                << "ns"
-                                << nss.ns()
-                                << "o"
-                                << documentToInsert
-                                << "ui"
-                                << *uuid)
+                                << "ns" << nss.ns() << "o" << documentToInsert << "ui" << *uuid)
                          : BSON("op"
                                 << "i"
-                                << "ns"
-                                << nss.ns()
-                                << "o"
-                                << documentToInsert);
+                                << "ns" << nss.ns() << "o" << documentToInsert);
     return BSON("applyOps" << BSON_ARRAY(insertOp));
 }
 
@@ -266,7 +248,7 @@ TEST_F(ApplyOpsTest, AtomicApplyOpsInsertWithUuidIntoCollectionWithOtherUuid) {
     // Collection has a different UUID.
     CollectionOptions collectionOptions;
     collectionOptions.uuid = UUID::gen();
-    ASSERT_NOT_EQUALS(applyOpsUuid, collectionOptions.uuid);
+    ASSERT_NOT_EQUALS(applyOpsUuid, *collectionOptions.uuid);
     ASSERT_OK(_storage->createCollection(opCtx.get(), nss, collectionOptions));
 
     // The applyOps returns a NamespaceNotFound error because of the failed UUID lookup
@@ -353,7 +335,7 @@ TEST_F(ApplyOpsTest, ApplyOpsPropagatesOplogApplicationMode) {
  */
 OplogEntry makeOplogEntry(OpTypeEnum opType, const BSONObj& oField) {
     return OplogEntry(OpTime(Timestamp(1, 1), 1),  // optime
-                      1LL,                         // hash
+                      boost::none,                 // hash
                       opType,                      // op type
                       NamespaceString("a.a"),      // namespace
                       boost::none,                 // uuid
@@ -396,55 +378,37 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
     auto ui1 = UUID::gen();
     auto op1 = BSON("op"
                     << "i"
-                    << "ns"
-                    << ns1.ns()
-                    << "ui"
-                    << ui1
-                    << "o"
-                    << BSON("_id" << 1));
+                    << "ns" << ns1.ns() << "ui" << ui1 << "o" << BSON("_id" << 1));
 
     NamespaceString ns2("test.b");
     auto ui2 = UUID::gen();
     auto op2 = BSON("op"
                     << "i"
-                    << "ns"
-                    << ns2.ns()
-                    << "ui"
-                    << ui2
-                    << "o"
-                    << BSON("_id" << 2));
+                    << "ns" << ns2.ns() << "ui" << ui2 << "o" << BSON("_id" << 2));
 
     NamespaceString ns3("test.c");
     auto ui3 = UUID::gen();
     auto op3 = BSON("op"
                     << "u"
-                    << "ns"
-                    << ns3.ns()
-                    << "ui"
-                    << ui3
-                    << "b"
-                    << true
-                    << "o"
-                    << BSON("x" << 1)
-                    << "o2"
-                    << BSON("_id" << 3));
+                    << "ns" << ns3.ns() << "ui" << ui3 << "b" << true << "o" << BSON("x" << 1)
+                    << "o2" << BSON("_id" << 3));
 
     auto oplogEntry =
         makeOplogEntry(OpTypeEnum::kCommand, BSON("applyOps" << BSON_ARRAY(op1 << op2 << op3)));
 
     auto operations = ApplyOps::extractOperations(oplogEntry);
-    ASSERT_EQUALS(3U, operations.size()) << "Unexpected number of operations extracted: "
-                                         << oplogEntry.toBSON();
+    ASSERT_EQUALS(3U, operations.size())
+        << "Unexpected number of operations extracted: " << oplogEntry.toBSON();
 
     // Check extracted CRUD operations.
     auto it = operations.cbegin();
     {
         ASSERT(operations.cend() != it);
         const auto& operation1 = *(it++);
-        ASSERT(OpTypeEnum::kInsert == operation1.getOpType()) << "Unexpected op type: "
-                                                              << operation1.toBSON();
+        ASSERT(OpTypeEnum::kInsert == operation1.getOpType())
+            << "Unexpected op type: " << operation1.toBSON();
         ASSERT_EQUALS(ui1, *operation1.getUuid());
-        ASSERT_EQUALS(ns1, operation1.getNamespace());
+        ASSERT_EQUALS(ns1, operation1.getNss());
         ASSERT_BSONOBJ_EQ(BSON("_id" << 1), operation1.getOperationToApply());
 
         // OpTime of CRUD operation should match applyOps.
@@ -454,10 +418,10 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
     {
         ASSERT(operations.cend() != it);
         const auto& operation2 = *(it++);
-        ASSERT(OpTypeEnum::kInsert == operation2.getOpType()) << "Unexpected op type: "
-                                                              << operation2.toBSON();
+        ASSERT(OpTypeEnum::kInsert == operation2.getOpType())
+            << "Unexpected op type: " << operation2.toBSON();
         ASSERT_EQUALS(ui2, *operation2.getUuid());
-        ASSERT_EQUALS(ns2, operation2.getNamespace());
+        ASSERT_EQUALS(ns2, operation2.getNss());
         ASSERT_BSONOBJ_EQ(BSON("_id" << 2), operation2.getOperationToApply());
 
         // OpTime of CRUD operation should match applyOps.
@@ -467,10 +431,10 @@ TEST_F(ApplyOpsTest, ExtractOperationsReturnsOperationsWithSameOpTimeAsApplyOps)
     {
         ASSERT(operations.cend() != it);
         const auto& operation3 = *(it++);
-        ASSERT(OpTypeEnum::kUpdate == operation3.getOpType()) << "Unexpected op type: "
-                                                              << operation3.toBSON();
+        ASSERT(OpTypeEnum::kUpdate == operation3.getOpType())
+            << "Unexpected op type: " << operation3.toBSON();
         ASSERT_EQUALS(ui3, *operation3.getUuid());
-        ASSERT_EQUALS(ns3, operation3.getNamespace());
+        ASSERT_EQUALS(ns3, operation3.getNss());
         ASSERT_BSONOBJ_EQ(BSON("x" << 1), operation3.getOperationToApply());
 
         auto optionalUpsertBool = operation3.getUpsert();
@@ -496,9 +460,7 @@ TEST_F(ApplyOpsTest, ApplyOpsFailsToDropAdmin) {
 
     auto dropDatabaseOp = BSON("op"
                                << "c"
-                               << "ns"
-                               << nss.getCommandNS().ns()
-                               << "o"
+                               << "ns" << nss.getCommandNS().ns() << "o"
                                << BSON("dropDatabase" << 1));
 
     auto dropDatabaseCmdObj = BSON("applyOps" << BSON_ARRAY(dropDatabaseOp));
