@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -40,9 +39,10 @@
 #include "mongo/db/repl/base_cloner_test_fixture.h"
 #include "mongo/db/repl/database_cloner.h"
 #include "mongo/db/repl/storage_interface.h"
+#include "mongo/dbtests/mock/mock_dbclient_connection.h"
 #include "mongo/unittest/task_executor_proxy.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/mongoutils/str.h"
+#include "mongo/util/str.h"
 #include "mongo/util/uuid.h"
 
 namespace {
@@ -54,13 +54,18 @@ using namespace unittest;
 const std::string dbname("db");
 
 struct CollectionCloneInfo {
-    CollectionMockStats stats;
+    std::shared_ptr<CollectionMockStats> stats = std::make_shared<CollectionMockStats>();
     CollectionBulkLoaderMock* loader = nullptr;
     Status status{ErrorCodes::NotYetInitialized, ""};
 };
 
 class DatabaseClonerTest : public BaseClonerTest {
 public:
+    DatabaseClonerTest() {
+        _options1.uuid = UUID::gen();
+        _options2.uuid = UUID::gen();
+        _options3.uuid = UUID::gen();
+    }
     void clear() override;
     BaseCloner* getCloner() const override;
 
@@ -79,6 +84,11 @@ protected:
 
     std::map<NamespaceString, CollectionCloneInfo> _collections;
     std::unique_ptr<DatabaseCloner> _databaseCloner;
+    CollectionOptions _options1;
+    CollectionOptions _options2;
+    CollectionOptions _options3;
+    DatabaseCloner::StartCollectionClonerFn _startCollectionCloner;
+    std::unique_ptr<MockRemoteDBServer> _mockServer;
 };
 
 void DatabaseClonerTest::setUp() {
@@ -93,24 +103,39 @@ void DatabaseClonerTest::setUp() {
                                           storageInterface.get(),
                                           makeCollectionWorkClosure(),
                                           makeSetStatusClosure());
-    _databaseCloner->setScheduleDbWorkFn_forTest(
-        [this](const executor::TaskExecutor::CallbackFn& work) {
-            return getExecutor().scheduleWork(work);
+    _databaseCloner->setScheduleDbWorkFn_forTest([this](executor::TaskExecutor::CallbackFn work) {
+        return getExecutor().scheduleWork(std::move(work));
+    });
+
+    _mockServer = stdx::make_unique<MockRemoteDBServer>(target.toString());
+    _mockServer->assignCollectionUuid("db.a", *_options1.uuid);
+    _mockServer->assignCollectionUuid("db.b", *_options2.uuid);
+    _mockServer->assignCollectionUuid("db.c", *_options3.uuid);
+    _startCollectionCloner = [this](CollectionCloner& cloner) {
+        cloner.setCreateClientFn_forTest([&cloner, this]() {
+            return std::unique_ptr<DBClientConnection>(
+                new MockDBClientConnection(_mockServer.get()));
         });
+        return cloner.startup();
+    };
+    _databaseCloner->setStartCollectionClonerFn(_startCollectionCloner);
 
     storageInterface->createCollectionForBulkFn =
         [this](const NamespaceString& nss,
                const CollectionOptions& options,
                const BSONObj& idIndexSpec,
-               const std::vector<BSONObj>& secondaryIndexSpecs) {
-            const auto collInfo = &_collections[nss];
-            (collInfo->loader = new CollectionBulkLoaderMock(&collInfo->stats))
-                ->init(secondaryIndexSpecs)
-                .transitional_ignore();
+               const std::vector<BSONObj>& secondaryIndexSpecs)
+        -> StatusWith<std::unique_ptr<CollectionBulkLoaderMock>> {
+        const auto collInfo = &_collections[nss];
 
-            return StatusWith<std::unique_ptr<CollectionBulkLoader>>(
-                std::unique_ptr<CollectionBulkLoader>(collInfo->loader));
-        };
+        auto localLoader = std::make_unique<CollectionBulkLoaderMock>(collInfo->stats);
+        auto status = localLoader->init(secondaryIndexSpecs);
+        if (!status.isOK())
+            return status;
+        collInfo->loader = localLoader.get();
+
+        return std::move(localLoader);
+    };
 }
 
 void DatabaseClonerTest::tearDown() {
@@ -208,10 +233,9 @@ public:
                                                    ShouldFailRequestFn shouldFailRequest)
         : unittest::TaskExecutorProxy(executor), _shouldFailRequest(shouldFailRequest) {}
 
-    StatusWith<CallbackHandle> scheduleRemoteCommand(
-        const executor::RemoteCommandRequest& request,
-        const RemoteCommandCallbackFn& cb,
-        const transport::BatonHandle& baton = nullptr) override {
+    StatusWith<CallbackHandle> scheduleRemoteCommand(const executor::RemoteCommandRequest& request,
+                                                     const RemoteCommandCallbackFn& cb,
+                                                     const BatonHandle& baton = nullptr) override {
         if (_shouldFailRequest(request)) {
             return Status(ErrorCodes::OperationFailed, "failed to schedule remote command");
         }
@@ -226,7 +250,7 @@ TEST_F(DatabaseClonerTest,
        DatabaseClonerReturnsScheduleErrorOnFailingToScheduleListCollectionsCommand) {
     TaskExecutorWithFailureInScheduleRemoteCommand executorProxy(
         &getExecutor(), [](const executor::RemoteCommandRequest& request) {
-            return str::equals("listCollections", request.cmdObj.firstElementFieldName());
+            return request.cmdObj.firstElementFieldNameStringData() == "listCollections";
         });
 
     DatabaseCloner databaseCloner(&executorProxy,
@@ -311,8 +335,7 @@ TEST_F(DatabaseClonerTest, InvalidListCollectionsFilter) {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(BSON("ok" << 0 << "errmsg"
                                          << "unknown operator"
-                                         << "code"
-                                         << ErrorCodes::BadValue));
+                                         << "code" << ErrorCodes::BadValue));
     }
 
     ASSERT_EQUALS(ErrorCodes::BadValue, getStatus().code());
@@ -367,16 +390,13 @@ TEST_F(DatabaseClonerTest, ListCollectionsPredicate) {
 
     const std::vector<BSONObj> sourceInfos = {BSON("name"
                                                    << "a"
-                                                   << "options"
-                                                   << BSONObj()),
+                                                   << "options" << _options1.toBSON()),
                                               BSON("name"
                                                    << "b"
-                                                   << "options"
-                                                   << BSONObj()),
+                                                   << "options" << _options2.toBSON()),
                                               BSON("name"
                                                    << "c"
-                                                   << "options"
-                                                   << BSONObj())};
+                                                   << "options" << _options3.toBSON())};
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(createListCollectionsResponse(
@@ -401,12 +421,10 @@ TEST_F(DatabaseClonerTest, ListCollectionsMultipleBatches) {
 
     const std::vector<BSONObj> sourceInfos = {BSON("name"
                                                    << "a"
-                                                   << "options"
-                                                   << BSONObj()),
+                                                   << "options" << _options1.toBSON()),
                                               BSON("name"
                                                    << "b"
-                                                   << "options"
-                                                   << BSONObj())};
+                                                   << "options" << _options2.toBSON())};
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(createListCollectionsResponse(1, BSON_ARRAY(sourceInfos[0])));
@@ -449,7 +467,7 @@ TEST_F(DatabaseClonerTest, CollectionInfoNameFieldMissing) {
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(
-            createListCollectionsResponse(0, BSON_ARRAY(BSON("options" << BSONObj()))));
+            createListCollectionsResponse(0, BSON_ARRAY(BSON("options" << _options1.toBSON()))));
     }
 
     ASSERT_EQUALS(ErrorCodes::FailedToParse, getStatus().code());
@@ -467,7 +485,7 @@ TEST_F(DatabaseClonerTest, CollectionInfoNameNotAString) {
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(createListCollectionsResponse(
-            0, BSON_ARRAY(BSON("name" << 123 << "options" << BSONObj()))));
+            0, BSON_ARRAY(BSON("name" << 123 << "options" << _options1.toBSON()))));
     }
 
     ASSERT_EQUALS(ErrorCodes::TypeMismatch, getStatus().code());
@@ -484,11 +502,11 @@ TEST_F(DatabaseClonerTest, CollectionInfoNameEmpty) {
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << ""
-                                                                             << "options"
-                                                                             << BSONObj()))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << ""
+                                                          << "options" << _options1.toBSON()))));
     }
 
     ASSERT_EQUALS(ErrorCodes::BadValue, getStatus().code());
@@ -505,18 +523,17 @@ TEST_F(DatabaseClonerTest, CollectionInfoNameDuplicate) {
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << "a"
-                                                                             << "options"
-                                                                             << BSONObj())
-                                                                        << BSON("name"
-                                                                                << "a"
-                                                                                << "options"
-                                                                                << BSONObj()))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << "a"
+                                                          << "options" << _options1.toBSON())
+                                                     << BSON("name"
+                                                             << "a"
+                                                             << "options" << _options2.toBSON()))));
     }
 
-    ASSERT_EQUALS(ErrorCodes::DuplicateKey, getStatus().code());
+    ASSERT_EQUALS(51005, getStatus().code());
     ASSERT_STRING_CONTAINS(getStatus().reason(), "duplicate collection name 'a'");
     ASSERT_FALSE(_databaseCloner->isActive());
     ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
@@ -549,11 +566,11 @@ TEST_F(DatabaseClonerTest, CollectionInfoOptionsNotAnObject) {
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << "a"
-                                                                             << "options"
-                                                                             << 123))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << "a"
+                                                          << "options" << 123))));
     }
 
     ASSERT_EQUALS(ErrorCodes::TypeMismatch, getStatus().code());
@@ -568,17 +585,36 @@ TEST_F(DatabaseClonerTest, InvalidCollectionOptions) {
     ASSERT_OK(_databaseCloner->startup());
     ASSERT_EQUALS(DatabaseCloner::State::kRunning, _databaseCloner->getState_forTest());
 
+    {
+        executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
+        processNetworkResponse(createListCollectionsResponse(
+            0,
+            BSON_ARRAY(BSON("name"
+                            << "a"
+                            << "options" << BSON("storageEngine" << 1)))));
+    }
+
+    ASSERT_EQUALS(ErrorCodes::BadValue, getStatus().code());
+    ASSERT_FALSE(_databaseCloner->isActive());
+    ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
+}
+
+TEST_F(DatabaseClonerTest, InvalidMissingUUID) {
+    ASSERT_EQUALS(DatabaseCloner::State::kPreStart, _databaseCloner->getState_forTest());
+
+    ASSERT_OK(_databaseCloner->startup());
+    ASSERT_EQUALS(DatabaseCloner::State::kRunning, _databaseCloner->getState_forTest());
+
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(
             createListCollectionsResponse(0,
                                           BSON_ARRAY(BSON("name"
                                                           << "a"
-                                                          << "options"
-                                                          << BSON("storageEngine" << 1)))));
+                                                          << "options" << BSONObj()))));
     }
 
-    ASSERT_EQUALS(ErrorCodes::BadValue, getStatus().code());
+    ASSERT_EQUALS(50953, getStatus().code());
     ASSERT_FALSE(_databaseCloner->isActive());
     ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
 }
@@ -593,8 +629,8 @@ TEST_F(DatabaseClonerTest, DatabaseClonerResendsListCollectionsRequestOnRetriabl
     executor::NetworkInterfaceMock::InNetworkGuard guard(net);
 
     // Respond to first listCollections request with a retriable error.
-    assertRemoteCommandNameEquals("listCollections",
-                                  net->scheduleErrorResponse(Status(ErrorCodes::HostNotFound, "")));
+    assertRemoteCommandNameEquals(
+        "listCollections", net->scheduleErrorResponse(Status(ErrorCodes::HostUnreachable, "")));
     net->runReadyNetworkOperations();
 
     // DatabaseCloner stays active because it resends the listCollections request.
@@ -608,16 +644,15 @@ TEST_F(DatabaseClonerTest, DatabaseClonerResendsListCollectionsRequestOnRetriabl
 }
 
 TEST_F(DatabaseClonerTest, ListCollectionsReturnsEmptyCollectionName) {
-    _databaseCloner =
-        stdx::make_unique<DatabaseCloner>(&getExecutor(),
-                                          dbWorkThreadPool.get(),
-                                          target,
-                                          dbname,
-                                          BSONObj(),
-                                          DatabaseCloner::ListCollectionsPredicateFn(),
-                                          storageInterface.get(),
-                                          makeCollectionWorkClosure(),
-                                          makeSetStatusClosure());
+    _databaseCloner = std::make_unique<DatabaseCloner>(&getExecutor(),
+                                                       dbWorkThreadPool.get(),
+                                                       target,
+                                                       dbname,
+                                                       BSONObj(),
+                                                       DatabaseCloner::ListCollectionsPredicateFn(),
+                                                       storageInterface.get(),
+                                                       makeCollectionWorkClosure(),
+                                                       makeSetStatusClosure());
     ASSERT_EQUALS(DatabaseCloner::State::kPreStart, _databaseCloner->getState_forTest());
 
     ASSERT_OK(_databaseCloner->startup());
@@ -625,11 +660,11 @@ TEST_F(DatabaseClonerTest, ListCollectionsReturnsEmptyCollectionName) {
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << ""
-                                                                             << "options"
-                                                                             << BSONObj()))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << ""
+                                                          << "options" << BSONObj()))));
     }
 
     ASSERT_EQUALS(ErrorCodes::BadValue, getStatus().code());
@@ -638,37 +673,6 @@ TEST_F(DatabaseClonerTest, ListCollectionsReturnsEmptyCollectionName) {
     ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
 }
 
-TEST_F(DatabaseClonerTest, DatabaseClonerAcceptsCollectionOptionsContainUuid) {
-    ASSERT_EQUALS(DatabaseCloner::State::kPreStart, _databaseCloner->getState_forTest());
-
-    ASSERT_OK(_databaseCloner->startup());
-    ASSERT_EQUALS(DatabaseCloner::State::kRunning, _databaseCloner->getState_forTest());
-
-    bool collectionClonerStarted = false;
-    _databaseCloner->setStartCollectionClonerFn(
-        [&collectionClonerStarted](CollectionCloner& cloner) {
-            collectionClonerStarted = true;
-            return cloner.startup();
-        });
-
-    {
-        executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        CollectionOptions options;
-        options.uuid = UUID::gen();
-        processNetworkResponse(
-            createListCollectionsResponse(0,
-                                          BSON_ARRAY(BSON("name"
-                                                          << "a"
-                                                          << "options"
-                                                          << options.toBSON()))));
-    }
-
-    ASSERT_EQUALS(getDetectableErrorStatus(), getStatus());
-    ASSERT_TRUE(collectionClonerStarted);
-    ASSERT_TRUE(_databaseCloner->isActive());
-    ASSERT_EQUALS(DatabaseCloner::State::kRunning, _databaseCloner->getState_forTest());
-}
-
 TEST_F(DatabaseClonerTest, StartFirstCollectionClonerFailed) {
     ASSERT_EQUALS(DatabaseCloner::State::kPreStart, _databaseCloner->getState_forTest());
 
@@ -682,11 +686,11 @@ TEST_F(DatabaseClonerTest, StartFirstCollectionClonerFailed) {
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << "a"
-                                                                             << "options"
-                                                                             << BSONObj()))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << "a"
+                                                          << "options" << _options1.toBSON()))));
     }
 
     ASSERT_EQUALS(ErrorCodes::OperationFailed, getStatus().code());
@@ -703,28 +707,27 @@ TEST_F(DatabaseClonerTest, StartSecondCollectionClonerFailed) {
     const Status errStatus{ErrorCodes::OperationFailed,
                            "StartSecondCollectionClonerFailed injected failure."};
 
-    _databaseCloner->setStartCollectionClonerFn([errStatus](CollectionCloner& cloner) -> Status {
-        if (cloner.getSourceNamespace().coll() == "b") {
-            return errStatus;
-        }
-        return cloner.startup();
-    });
+    _databaseCloner->setStartCollectionClonerFn(
+        [errStatus, this](CollectionCloner& cloner) -> Status {
+            if (cloner.getSourceNamespace().coll() == "b") {
+                return errStatus;
+            }
+            return _startCollectionCloner(cloner);
+        });
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createListCollectionsResponse(0,
-                                                             BSON_ARRAY(BSON("name"
-                                                                             << "a"
-                                                                             << "options"
-                                                                             << BSONObj())
-                                                                        << BSON("name"
-                                                                                << "b"
-                                                                                << "options"
-                                                                                << BSONObj()))));
+        processNetworkResponse(
+            createListCollectionsResponse(0,
+                                          BSON_ARRAY(BSON("name"
+                                                          << "a"
+                                                          << "options" << _options1.toBSON())
+                                                     << BSON("name"
+                                                             << "b"
+                                                             << "options" << _options2.toBSON()))));
 
         processNetworkResponse(createCountResponse(0));
         processNetworkResponse(createListIndexesResponse(0, BSON_ARRAY(idIndexSpec)));
-        processNetworkResponse(createCursorResponse(0, BSONArray()));
     }
     _databaseCloner->join();
     ASSERT_FALSE(_databaseCloner->isActive());
@@ -747,19 +750,14 @@ TEST_F(DatabaseClonerTest, ShutdownCancelsCollectionCloning) {
                                           0,
                                           BSON_ARRAY(BSON("name"
                                                           << "a"
-                                                          << "options"
-                                                          << BSONObj())
-                                                     << BSON("name"
-                                                             << "b"
-                                                             << "options"
-                                                             << BSONObj())))));
+                                                          << "options" << _options1.toBSON())))));
         net->runReadyNetworkOperations();
 
         // CollectionCloner sends collection count request on startup.
         // Blackhole count request to leave collection cloner active.
         auto noi = net->getNextReadyRequest();
         assertRemoteCommandNameEquals("count", noi->getRequest());
-        ASSERT_EQUALS("a", noi->getRequest().cmdObj.firstElement().String());
+        ASSERT_EQUALS(*_options1.uuid, UUID::parse(noi->getRequest().cmdObj.firstElement()));
         net->blackHole(noi);
     }
 
@@ -773,9 +771,8 @@ TEST_F(DatabaseClonerTest, ShutdownCancelsCollectionCloning) {
     ASSERT_FALSE(_databaseCloner->isActive());
     ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
 
-    // This is the error code from attempting to start up the last (of 2) collection cloner which
-    // was shut down before it was ever started.
-    ASSERT_EQUALS(ErrorCodes::ShutdownInProgress, getStatus());
+    // We do not need to attempt a subsequent collection clone to be notified of an error.
+    ASSERT_EQUALS(ErrorCodes::InitialSyncFailure, getStatus());
 }
 
 TEST_F(DatabaseClonerTest, FirstCollectionListIndexesFailed) {
@@ -786,12 +783,10 @@ TEST_F(DatabaseClonerTest, FirstCollectionListIndexesFailed) {
 
     const std::vector<BSONObj> sourceInfos = {BSON("name"
                                                    << "a"
-                                                   << "options"
-                                                   << BSONObj()),
+                                                   << "options" << _options1.toBSON()),
                                               BSON("name"
                                                    << "b"
-                                                   << "options"
-                                                   << BSONObj())};
+                                                   << "options" << _options2.toBSON())};
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(
@@ -807,29 +802,29 @@ TEST_F(DatabaseClonerTest, FirstCollectionListIndexesFailed) {
         processNetworkResponse(createCountResponse(0));
         processNetworkResponse(BSON("ok" << 0 << "errmsg"
                                          << "fake message"
-                                         << "code"
-                                         << ErrorCodes::CursorNotFound));
+                                         << "code" << ErrorCodes::CursorNotFound));
 
         processNetworkResponse(createCountResponse(0));
         processNetworkResponse(createListIndexesResponse(0, BSON_ARRAY(idIndexSpec)));
-        processNetworkResponse(createCursorResponse(0, BSONArray()));
     }
     _databaseCloner->join();
     ASSERT_EQ(getStatus().code(), ErrorCodes::InitialSyncFailure);
     ASSERT_FALSE(_databaseCloner->isActive());
     ASSERT_EQUALS(DatabaseCloner::State::kComplete, _databaseCloner->getState_forTest());
 
-    ASSERT_EQUALS(2U, _collections.size());
+    ASSERT_EQUALS(1U, _collections.size());
 
+    // We have attempted, and failed, to clone the first collection.
     auto collInfo = _collections[NamespaceString{"db.a"}];
     ASSERT_EQUALS(ErrorCodes::CursorNotFound, collInfo.status.code());
-    auto stats = collInfo.stats;
+    auto stats = *collInfo.stats;
     stats.insertCount = 0;
     stats.commitCalled = false;
 
+    // We have not attempted to clone the second collection.
     collInfo = _collections[NamespaceString{"db.b"}];
-    ASSERT_OK(collInfo.status);
-    stats = collInfo.stats;
+    ASSERT_EQUALS(ErrorCodes::NotYetInitialized, collInfo.status.code());
+    stats = *collInfo.stats;
     stats.insertCount = 0;
     stats.commitCalled = true;
 }
@@ -842,12 +837,10 @@ TEST_F(DatabaseClonerTest, CreateCollections) {
 
     const std::vector<BSONObj> sourceInfos = {BSON("name"
                                                    << "a"
-                                                   << "options"
-                                                   << BSONObj()),
+                                                   << "options" << _options1.toBSON()),
                                               BSON("name"
                                                    << "b"
-                                                   << "options"
-                                                   << BSONObj())};
+                                                   << "options" << _options2.toBSON())};
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(
@@ -864,22 +857,12 @@ TEST_F(DatabaseClonerTest, CreateCollections) {
         processNetworkResponse(createListIndexesResponse(0, BSON_ARRAY(idIndexSpec)));
     }
     ASSERT_TRUE(_databaseCloner->isActive());
-    {
-        executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createCursorResponse(0, BSONArray()));
-    }
-    ASSERT_TRUE(_databaseCloner->isActive());
 
     {
         executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
         processNetworkResponse(createCountResponse(0));
         processNetworkResponse(createListIndexesResponse(0, BSON_ARRAY(idIndexSpec)));
     }
-    ASSERT_TRUE(_databaseCloner->isActive());
-    {
-        executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-        processNetworkResponse(createCursorResponse(0, BSONArray()));
-    }
 
     _databaseCloner->join();
     ASSERT_OK(getStatus());
@@ -890,13 +873,13 @@ TEST_F(DatabaseClonerTest, CreateCollections) {
 
     auto collInfo = _collections[NamespaceString{"db.a"}];
     ASSERT_OK(collInfo.status);
-    auto stats = collInfo.stats;
+    auto stats = *collInfo.stats;
     stats.insertCount = 0;
     stats.commitCalled = true;
 
     collInfo = _collections[NamespaceString{"db.b"}];
     ASSERT_OK(collInfo.status);
-    stats = collInfo.stats;
+    stats = *collInfo.stats;
     stats.insertCount = 0;
     stats.commitCalled = true;
 }
