@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -31,21 +30,20 @@
 #include "mongo/platform/basic.h"
 
 #include "mongo/bson/simple_bsonelement_comparator.h"
-#include "mongo/db/catalog/collection_catalog_entry.h"
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/health_log.h"
 #include "mongo/db/catalog/index_catalog.h"
-#include "mongo/db/catalog/uuid_catalog.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/repl/dbcheck.h"
-#include "mongo/db/repl/dbcheck.h"
 #include "mongo/db/repl/dbcheck_gen.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/storage/durable_catalog.h"
 
 namespace mongo {
 
@@ -124,7 +122,7 @@ std::unique_ptr<HealthLogEntry> dbCheckHealthLogEntry(const NamespaceString& nss
                                                       OplogEntriesEnum operation,
                                                       const BSONObj& data) {
     auto entry = stdx::make_unique<HealthLogEntry>();
-    entry->setNamespace(nss);
+    entry->setNss(nss);
     entry->setTimestamp(Date_t::now());
     entry->setSeverity(severity);
     entry->setScope(ScopeEnum::Cluster);
@@ -133,7 +131,7 @@ std::unique_ptr<HealthLogEntry> dbCheckHealthLogEntry(const NamespaceString& nss
     entry->setData(data);
     return entry;
 }
-}
+}  // namespace
 
 /**
  * Get an error message if the check fails.
@@ -162,14 +160,9 @@ std::unique_ptr<HealthLogEntry> dbCheckBatchEntry(const NamespaceString& nss,
                                                   const repl::OpTime& optime) {
     auto hashes = expectedFound(expectedHash, foundHash);
 
-    auto data =
-        BSON("success" << true << "count" << count << "bytes" << bytes << "md5" << hashes.second
-                       << "minKey"
-                       << minKey.elem()
-                       << "maxKey"
-                       << maxKey.elem()
-                       << "optime"
-                       << optime);
+    auto data = BSON("success" << true << "count" << count << "bytes" << bytes << "md5"
+                               << hashes.second << "minKey" << minKey.elem() << "maxKey"
+                               << maxKey.elem() << "optime" << optime);
 
     auto severity = hashes.first ? SeverityEnum::Info : SeverityEnum::Error;
     std::string msg =
@@ -190,7 +183,7 @@ DbCheckHasher::DbCheckHasher(OperationContext* opCtx,
     md5_init(&_state);
 
     // Get the _id index.
-    IndexDescriptor* desc = collection->getIndexCatalog()->findIdIndex(opCtx);
+    const IndexDescriptor* desc = collection->getIndexCatalog()->findIdIndex(opCtx);
 
     uassert(ErrorCodes::IndexNotFound, "dbCheck needs _id index", desc);
 
@@ -239,6 +232,33 @@ std::string hashCollectionInfo(const DbCheckCollectionInformation& info) {
     return digestToString(digest);
 }
 
+std::pair<boost::optional<UUID>, boost::optional<UUID>> getPrevAndNextUUIDs(
+    OperationContext* opCtx, Collection* collection) {
+    const CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
+    const UUID uuid = *collection->uuid();
+
+    std::vector<CollectionUUID> collectionUUIDs =
+        catalog.getAllCollectionUUIDsFromDb(collection->ns().db());
+    auto uuidIt = std::find(collectionUUIDs.begin(), collectionUUIDs.end(), uuid);
+    invariant(uuidIt != collectionUUIDs.end());
+
+    auto prevIt = std::prev(uuidIt);
+    auto nextIt = std::next(uuidIt);
+
+    boost::optional<UUID> prevUUID;
+    boost::optional<UUID> nextUUID;
+
+    if (prevIt != collectionUUIDs.end()) {
+        prevUUID = *prevIt;
+    }
+
+    if (nextIt != collectionUUIDs.end()) {
+        nextUUID = *nextIt;
+    }
+
+    return std::make_pair(prevUUID, nextUUID);
+}
+
 std::unique_ptr<HealthLogEntry> dbCheckCollectionEntry(const NamespaceString& nss,
                                                        const UUID& uuid,
                                                        const DbCheckCollectionInformation& expected,
@@ -258,19 +278,9 @@ std::unique_ptr<HealthLogEntry> dbCheckCollectionEntry(const NamespaceString& ns
     std::string msg =
         "dbCheck collection " + (match ? std::string("consistent") : std::string("inconsistent"));
     auto data = BSON("success" << true << "uuid" << uuid.toString() << "found" << true << "name"
-                               << names.second
-                               << "prev"
-                               << prevs.second
-                               << "next"
-                               << nexts.second
-                               << "indexes"
-                               << indices.second
-                               << "options"
-                               << options.second
-                               << "md5"
-                               << md5s.second
-                               << "optime"
-                               << optime);
+                               << names.second << "prev" << prevs.second << "next" << nexts.second
+                               << "indexes" << indices.second << "options" << options.second
+                               << "md5" << md5s.second << "optime" << optime);
 
     return dbCheckHealthLogEntry(nss, severity, msg, OplogEntriesEnum::Collection, data);
 }
@@ -348,14 +358,12 @@ std::vector<BSONObj> collectionIndexInfo(OperationContext* opCtx, Collection* co
     std::vector<std::string> names;
 
     // List the indices,
-    const auto* cce = collection->getCatalogEntry();
-    invariant(cce);
-
-    cce->getAllIndexes(opCtx, &names);
+    auto durableCatalog = DurableCatalog::get(opCtx);
+    durableCatalog->getAllIndexes(opCtx, collection->ns(), &names);
 
     // and get the info for each one.
     for (const auto& name : names) {
-        result.push_back(cce->getIndexSpec(opCtx, name));
+        result.push_back(durableCatalog->getIndexSpec(opCtx, collection->ns(), name));
     }
 
     auto comp = stdx::make_unique<SimpleBSONObjComparator>();
@@ -366,7 +374,7 @@ std::vector<BSONObj> collectionIndexInfo(OperationContext* opCtx, Collection* co
 }
 
 BSONObj collectionOptions(OperationContext* opCtx, Collection* collection) {
-    return collection->getCatalogEntry()->getCollectionOptions(opCtx).toBSON();
+    return DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, collection->ns()).toBSON();
 }
 
 AutoGetDbForDbCheck::AutoGetDbForDbCheck(OperationContext* opCtx, const NamespaceString& nss)
@@ -375,7 +383,7 @@ AutoGetDbForDbCheck::AutoGetDbForDbCheck(OperationContext* opCtx, const Namespac
 AutoGetCollectionForDbCheck::AutoGetCollectionForDbCheck(OperationContext* opCtx,
                                                          const NamespaceString& nss,
                                                          const OplogEntriesEnum& type)
-    : _agd(opCtx, nss), _collLock(opCtx->lockState(), nss.ns(), MODE_S) {
+    : _agd(opCtx, nss), _collLock(opCtx, nss, MODE_S) {
     std::string msg;
 
     _collection = _agd.getDb() ? _agd.getDb()->getCollection(opCtx, nss) : nullptr;
@@ -453,7 +461,7 @@ Status dbCheckDatabaseOnSecondary(OperationContext* opCtx,
                                   const DbCheckOplogCollection& entry) {
     // dbCheckCollectionResult-specific stuff.
     auto uuid = uassertStatusOK(UUID::parse(entry.getUuid().toString()));
-    auto collection = UUIDCatalog::get(opCtx).lookupCollectionByUUID(uuid);
+    auto collection = CollectionCatalog::get(opCtx).lookupCollectionByUUID(uuid);
 
     if (!collection) {
         Status status(ErrorCodes::NamespaceNotFound, "Could not find collection for dbCheck");
@@ -472,13 +480,15 @@ Status dbCheckDatabaseOnSecondary(OperationContext* opCtx,
     expected.collectionName = entry.getNss().coll().toString();
     found.collectionName = collection->ns().coll().toString();
 
+    auto prevAndNext = getPrevAndNextUUIDs(opCtx, collection);
+
     // found/expected previous UUID,
     expected.prev = entry.getPrev();
-    found.prev = UUIDCatalog::get(opCtx).prev(db, uuid);
+    found.prev = prevAndNext.first;
 
     // found/expected next UUID,
     expected.next = entry.getNext();
-    found.next = UUIDCatalog::get(opCtx).next(db, uuid);
+    found.next = prevAndNext.second;
 
     // found/expected indices,
     expected.indexes = entry.getIndexes();
@@ -494,7 +504,7 @@ Status dbCheckDatabaseOnSecondary(OperationContext* opCtx,
 
     return Status::OK();
 }
-}
+}  // namespace
 
 namespace repl {
 
@@ -506,7 +516,9 @@ Status dbCheckOplogCommand(OperationContext* opCtx,
                            const BSONElement& ui,
                            BSONObj& cmd,
                            const repl::OpTime& optime,
-                           OplogApplication::Mode mode) {
+                           const repl::OplogEntry& entry,
+                           OplogApplication::Mode mode,
+                           boost::optional<Timestamp> stableTimestampForRecovery) {
     auto type = OplogEntries_parse(IDLParserErrorContext("type"), cmd.getStringField("type"));
     IDLParserErrorContext ctx("o");
 
