@@ -381,7 +381,7 @@ private:
      */
     void _startInitialSyncAttemptCallback(const executor::TaskExecutor::CallbackArgs& callbackArgs,
                                           std::uint32_t initialSyncAttempt,
-                                          std::uint32_t initialSyncMaxAttempts);
+                                          std::uint32_t initialSyncMaxAttempts) noexcept;
 
     /**
      * Callback to obtain sync source from sync source selector.
@@ -392,7 +392,7 @@ private:
     void _chooseSyncSourceCallback(const executor::TaskExecutor::CallbackArgs& callbackArgs,
                                    std::uint32_t chooseSyncSourceAttempt,
                                    std::uint32_t chooseSyncSourceMaxAttempts,
-                                   std::shared_ptr<OnCompletionGuard> onCompletionGuard);
+                                   std::shared_ptr<OnCompletionGuard> onCompletionGuard) noexcept;
 
     /**
      * This function does the following:
@@ -478,8 +478,9 @@ private:
     /**
      * Callback to obtain next batch of operations to apply.
      */
-    void _getNextApplierBatchCallback(const executor::TaskExecutor::CallbackArgs& callbackArgs,
-                                      std::shared_ptr<OnCompletionGuard> onCompletionGuard);
+    void _getNextApplierBatchCallback(
+        const executor::TaskExecutor::CallbackArgs& callbackArgs,
+        std::shared_ptr<OnCompletionGuard> onCompletionGuard) noexcept;
 
     /**
      * Callback for MultiApplier completion.
