@@ -34,6 +34,7 @@
 #include "mongo/db/repl/oplog_applier.h"
 
 #include "mongo/db/auth/authorization_session.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands/txn_cmds_gen.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
@@ -74,6 +75,14 @@ std::size_t OplogApplier::getBatchLimitOperations() {
 // static
 std::size_t OplogApplier::calculateBatchLimitBytes(OperationContext* opCtx,
                                                    StorageInterface* storageInterface) {
+    // We can't change the timestamp source within a write unit of work.
+    invariant(!opCtx->lockState()->inAWriteUnitOfWork());
+    // We're only reading oplog metadata, so the timestamp is not important.  If we read with the
+    // default (which is kLastApplied on secondaries), we may end up with a reader that is at
+    // kLastApplied.  If we then roll back, then when we reconstruct prepared transactions during
+    // rollback recovery we will be preparing transactions before the read timestamp, which triggers
+    // an assertion in WiredTiger.
+    ReadSourceScope readSourceScope(opCtx, RecoveryUnit::ReadSource::kNoTimestamp);
     auto oplogMaxSizeResult =
         storageInterface->getOplogMaxSize(opCtx, NamespaceString::kRsOplogNamespace);
     auto oplogMaxSize = fassert(40301, oplogMaxSizeResult);
