@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -255,7 +254,7 @@ TEST_F(SplitChunkTest, HashedKeyPatternNumberLongSplitKeys) {
     collResponse();
     chunkResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, HashedKeyPatternIntegerSplitKeys) {
@@ -293,7 +292,7 @@ TEST_F(SplitChunkTest, HashedKeyPatternIntegerSplitKeys) {
     chunkResponse();
     shardResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, HashedKeyPatternDoubleSplitKeys) {
@@ -331,7 +330,7 @@ TEST_F(SplitChunkTest, HashedKeyPatternDoubleSplitKeys) {
     chunkResponse();
     shardResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, HashedKeyPatternStringSplitKeys) {
@@ -375,7 +374,7 @@ TEST_F(SplitChunkTest, HashedKeyPatternStringSplitKeys) {
     chunkResponse();
     shardResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, ValidRangeKeyPatternSplitKeys) {
@@ -423,7 +422,7 @@ TEST_F(SplitChunkTest, ValidRangeKeyPatternSplitKeys) {
     collResponse();
     chunkResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, SplitChunkWithNoErrors) {
@@ -484,7 +483,7 @@ TEST_F(SplitChunkTest, SplitChunkWithNoErrors) {
     collResponse();
     chunkResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, AttemptSplitWithConfigsvrError) {
@@ -528,7 +527,7 @@ TEST_F(SplitChunkTest, AttemptSplitWithConfigsvrError) {
     collResponse();
     chunkResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, AttemptSplitOnNoDatabases) {
@@ -562,7 +561,7 @@ TEST_F(SplitChunkTest, AttemptSplitOnNoDatabases) {
     emptyResponse();
     emptyResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, AttemptSplitOnNoCollections) {
@@ -599,7 +598,7 @@ TEST_F(SplitChunkTest, AttemptSplitOnNoCollections) {
     emptyResponse();
     shardResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, AttemptSplitOnNoChunks) {
@@ -642,7 +641,7 @@ TEST_F(SplitChunkTest, AttemptSplitOnNoChunks) {
     collResponse();
     emptyResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, NoCollectionAfterSplit) {
@@ -686,7 +685,7 @@ TEST_F(SplitChunkTest, NoCollectionAfterSplit) {
     // Finally, give an empty response to a request regarding a find on the original collection.
     emptyResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 TEST_F(SplitChunkTest, NoChunksAfterSplit) {
@@ -736,7 +735,7 @@ TEST_F(SplitChunkTest, NoChunksAfterSplit) {
     collResponse();
     emptyResponse();
 
-    future.timed_get(kFutureTimeout);
+    future.default_timed_get();
 }
 
 }  // namespace
