@@ -1,6 +1,3 @@
-// wiredtiger_server_status.cpp
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -37,13 +34,15 @@
 #include "mongo/base/checked_cast.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/db_raii.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/scopeguard.h"
+
 
 namespace mongo {
 
@@ -70,8 +69,12 @@ BSONObj WiredTigerServerStatusSection::generateSection(OperationContext* opCtx,
     invariant(s);
     const string uri = "statistics:";
 
+    // Filter out unrelevant statistic fields.
+    std::vector<std::string> fieldsToIgnore = {"LSM"};
+
     BSONObjBuilder bob;
-    Status status = WiredTigerUtil::exportTableToBSON(s, uri, "statistics=(fast)", &bob);
+    Status status =
+        WiredTigerUtil::exportTableToBSON(s, uri, "statistics=(fast)", &bob, fieldsToIgnore);
     if (!status.isOK()) {
         bob.append("error", "unable to retrieve statistics");
         bob.append("code", static_cast<int>(status.code()));
@@ -80,7 +83,31 @@ BSONObj WiredTigerServerStatusSection::generateSection(OperationContext* opCtx,
 
     WiredTigerKVEngine::appendGlobalStats(bob);
 
+    WiredTigerUtil::appendSnapshotWindowSettings(_engine, session, &bob);
+
     return bob.obj();
 }
 
+OplogStonesServerStatusSection::OplogStonesServerStatusSection()
+    : ServerStatusSection("oplogTruncation") {}
+
+bool OplogStonesServerStatusSection::includeByDefault() const {
+    return true;
+}
+
+BSONObj OplogStonesServerStatusSection::generateSection(OperationContext* opCtx,
+                                                        const BSONElement& configElement) const {
+    BSONObjBuilder builder;
+    {
+        AutoGetCollectionForReadCommand ctx(opCtx, NamespaceString::kRsOplogNamespace);
+        Collection* oplogColl = ctx.getCollection();
+        if (oplogColl) {
+            auto oplogRS = checked_cast<WiredTigerRecordStore*>(oplogColl->getRecordStore());
+            oplogRS->getOplogTruncateStats(builder);
+        }
+    }
+    return builder.obj();
+}
+
+
 }  // namespace mongo
