@@ -1,6 +1,3 @@
-// client.cpp
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -32,7 +29,7 @@
 
 #include "mongo/platform/basic.h"
 
-#include "mongo/client/dbclientcursor.h"
+#include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/index_catalog.h"
@@ -43,8 +40,8 @@
 
 namespace ClientTests {
 
-using std::unique_ptr;
 using std::string;
+using std::unique_ptr;
 using std::vector;
 
 class Base {
@@ -99,9 +96,8 @@ public:
 };
 
 /**
- * Check that nIndexes is incremented correctly when an index builds (and that it is not
- * incremented when an index fails to build), system.indexes has an entry added (or not), and
- * system.namespaces has a doc added (or not).
+ * Check that nIndexes is incremented correctly when an index builds, and that it is not
+ * incremented when an index fails to build.
  */
 class BuildIndex : public Base {
 public:
@@ -110,7 +106,7 @@ public:
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
 
-        OldClientWriteContext ctx(&opCtx, ns());
+        dbtests::WriteContextForTests ctx(&opCtx, ns());
         DBDirectClient db(&opCtx);
 
         db.insert(ns(), BSON("x" << 1 << "y" << 2));
@@ -152,7 +148,8 @@ public:
 
         ASSERT_OK(dbtests::createIndex(&opCtx, ns(), BSON("a" << 1 << "b" << 1)));
 
-        unique_ptr<DBClientCursor> c = db.query(ns(), Query().sort(BSON("a" << 1 << "b" << 1)));
+        unique_ptr<DBClientCursor> c =
+            db.query(NamespaceString(ns()), Query().sort(BSON("a" << 1 << "b" << 1)));
         ASSERT_EQUALS(1111, c->itcount());
     }
 };
@@ -169,7 +166,8 @@ public:
             db.insert(ns(), BSON("i" << i));
         }
 
-        unique_ptr<DBClientCursor> c = db.query(ns(), Query().sort(BSON("i" << 1)));
+        unique_ptr<DBClientCursor> c =
+            db.query(NamespaceString(ns()), Query().sort(BSON("i" << 1)));
 
         BSONObj o = c->next();
         ASSERT(c->more());
@@ -409,4 +407,4 @@ public:
 };
 
 SuiteInstance<All> all;
-}
+}  // namespace ClientTests
