@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -42,6 +41,7 @@
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/log.h"
+#include "third_party/wiredtiger/wiredtiger.h"
 
 namespace mongo {
 namespace {
@@ -95,14 +95,19 @@ public:
     explicit Sized(T&&... args) {
         buffer.skip(sizeof(int32_t));
         append(args...);
-        DataView(buffer.buf()).write<LittleEndian<int32_t>>(buffer.len());
+        updateSize();
     }
 
     // Adds extra to the stored size. Use this to produce illegal messages.
     Sized&& addToSize(int32_t extra) && {
-        DataView(buffer.buf()).write<LittleEndian<int32_t>>(buffer.len() + extra);
+        updateSize(extra);
         return std::move(*this);
     }
+
+protected:
+    void updateSize(int32_t extra = 0) {
+        DataView(buffer.buf()).write<LittleEndian<int32_t>>(buffer.len() + extra);
+    }
 };
 
 // A Bytes that puts the standard message header at the front.
@@ -128,7 +133,25 @@ public:
     }
 
     OpMsgBytes&& addToSize(int32_t extra) && {
-        DataView(buffer.buf()).write<LittleEndian<int32_t>>(buffer.len() + extra);
+        updateSize(extra);
+        return std::move(*this);
+    }
+
+    OpMsgBytes&& appendChecksum() && {
+        // Reserve space at the end for the checksum.
+        append<uint32_t>(0);
+        updateSize();
+        // Checksum all bits except the checksum itself.
+        uint32_t checksum = wiredtiger_crc32c_func()(buffer.buf(), buffer.len() - 4);
+        // Write the checksum bits at the end.
+        auto checksumBits = DataView(buffer.buf() + buffer.len() - sizeof(checksum));
+        checksumBits.write<LittleEndian<uint32_t>>(checksum);
+        return std::move(*this);
+    }
+
+    OpMsgBytes&& appendChecksum(uint32_t checksum) && {
+        append(checksum);
+        updateSize();
         return std::move(*this);
     }
 };
@@ -159,45 +182,45 @@ const char kDocSequenceSection = 1;
 const uint32_t kNoFlags = 0;
 const uint32_t kHaveChecksum = 1;
 
-// CRC filler value
-const uint32_t kFakeCRC = 0;  // TODO will need to compute real crc when SERVER-28679 is done.
-
 TEST_F(OpMsgParser, SucceedsWithJustBody) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
-    }.parse();
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 0u);
 }
 
-TEST_F(OpMsgParser, IgnoresCrcIfPresent) {  // Until SERVER-28679 is done.
-    auto msg = OpMsgBytes{
-        kHaveChecksum,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
-        kFakeCRC,  // If not ignored, this would be read as a second body.
-    }.parse();
+TEST_F(OpMsgParser, SucceedsWithChecksum) {
+    auto msg = OpMsgBytes{kHaveChecksum,  //
+                          kBodySection,
+                          fromjson("{ping: 1}")}
+                   .appendChecksum()
+                   .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 0u);
 }
 
 TEST_F(OpMsgParser, SucceedsWithBodyThenSequence) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-            fromjson("{a: 2}"),
-        },
-    }.parse();
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+                fromjson("{a: 2}"),
+            },
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 1u);
@@ -208,17 +231,19 @@ TEST_F(OpMsgParser, SucceedsWithBodyThenSequence) {
 }
 
 TEST_F(OpMsgParser, SucceedsWithSequenceThenBody) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
 
-        kBodySection,
-        fromjson("{ping: 1}"),
-    }.parse();
+            kBodySection,
+            fromjson("{ping: 1}"),
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 1u);
@@ -228,22 +253,24 @@ TEST_F(OpMsgParser, SucceedsWithSequenceThenBody) {
 }
 
 TEST_F(OpMsgParser, SucceedsWithSequenceThenBodyThenSequence) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kDocSequenceSection,
-        Sized{
-            "empty",  //
-        },
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kDocSequenceSection,
+            Sized{
+                "empty",  //
+            },
 
-        kBodySection,
-        fromjson("{ping: 1}"),
+            kBodySection,
+            fromjson("{ping: 1}"),
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
-    }.parse();
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 2u);
@@ -255,22 +282,24 @@ TEST_F(OpMsgParser, SucceedsWithSequenceThenBodyThenSequence) {
 }
 
 TEST_F(OpMsgParser, SucceedsWithSequenceThenSequenceThenBody) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kDocSequenceSection,
-        Sized{
-            "empty",  //
-        },
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kDocSequenceSection,
+            Sized{
+                "empty",  //
+            },
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
 
-        kBodySection,
-        fromjson("{ping: 1}"),
-    }.parse();
+            kBodySection,
+            fromjson("{ping: 1}"),
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 2u);
@@ -282,22 +311,24 @@ TEST_F(OpMsgParser, SucceedsWithSequenceThenSequenceThenBody) {
 }
 
 TEST_F(OpMsgParser, SucceedsWithBodyThenSequenceThenSequence) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
 
-        kDocSequenceSection,
-        Sized{
-            "empty",  //
-        },
-    }.parse();
+            kDocSequenceSection,
+            Sized{
+                "empty",  //
+            },
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1}"));
     ASSERT_EQ(msg.sequences.size(), 2u);
@@ -383,17 +414,19 @@ TEST_F(OpMsgParser, FailsIfDuplicateSequenceWithBodyNested) {
 }
 
 TEST_F(OpMsgParser, SucceedsIfSequenceAndBodyHaveCommonPrefix) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{cursor: {ns: 'foo.bar', id: 1}}"),
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{cursor: {ns: 'foo.bar', id: 1}}"),
 
-        kDocSequenceSection,
-        Sized{
-            "cursor.firstBatch",  //
-            fromjson("{_id: 1}"),
-        },
-    }.parse();
+            kDocSequenceSection,
+            Sized{
+                "cursor.firstBatch",  //
+                fromjson("{_id: 1}"),
+            },
+        }
+            .parse();
 
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{cursor: {ns: 'foo.bar', id: 1}}"));
     ASSERT_EQ(msg.sequences.size(), 1u);
@@ -413,56 +446,63 @@ TEST_F(OpMsgParser, FailsIfUnknownSectionKind) {
 }
 
 TEST_F(OpMsgParser, FailsIfBodyTooBig) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
-    }.addToSize(-1);  // Shrink message so body extends past end.
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
+        }
+            .addToSize(-1);  // Shrink message so body extends past end.
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::InvalidBSON);
 }
 
 TEST_F(OpMsgParser, FailsIfBodyTooBigIntoChecksum) {
-    auto msg = OpMsgBytes{
-        kHaveChecksum,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
-        kFakeCRC,
-    }.addToSize(-1);  // Shrink message so body extends past end.
+    auto msg =
+        OpMsgBytes{
+            kHaveChecksum,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
+        }
+            .appendChecksum()
+            .addToSize(-1);  // Shrink message so body extends past end.
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::InvalidBSON);
 }
 
 TEST_F(OpMsgParser, FailsIfDocumentSequenceTooBig) {
-    auto msg = OpMsgBytes{
-        kNoFlags,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
+    auto msg =
+        OpMsgBytes{
+            kNoFlags,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
-    }.addToSize(-1);  // Shrink message so body extends past end.
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
+        }
+            .addToSize(-1);  // Shrink message so body extends past end.
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::Overflow);
 }
 
 TEST_F(OpMsgParser, FailsIfDocumentSequenceTooBigIntoChecksum) {
-    auto msg = OpMsgBytes{
-        kHaveChecksum,  //
-        kBodySection,
-        fromjson("{ping: 1}"),
+    auto msg =
+        OpMsgBytes{
+            kHaveChecksum,  //
+            kBodySection,
+            fromjson("{ping: 1}"),
 
-        kDocSequenceSection,
-        Sized{
-            "docs",  //
-            fromjson("{a: 1}"),
-        },
-
-        kFakeCRC,
-    }.addToSize(-1);  // Shrink message so body extends past end.
+            kDocSequenceSection,
+            Sized{
+                "docs",  //
+                fromjson("{a: 1}"),
+            },
+        }
+            .appendChecksum()
+            .addToSize(-1);  // Shrink message so body extends past end.
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::Overflow);
 }
@@ -477,7 +517,8 @@ TEST_F(OpMsgParser, FailsIfDocumentInSequenceTooBig) {
         Sized{
             "docs",  //
             fromjson("{a: 1}"),
-        }.addToSize(-1),  // Shrink sequence so document extends past end.
+        }
+            .addToSize(-1),  // Shrink sequence so document extends past end.
     };
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::InvalidBSON);
@@ -492,7 +533,8 @@ TEST_F(OpMsgParser, FailsIfNameOfDocumentSequenceTooBig) {
         kDocSequenceSection,
         Sized{
             "foo",
-        }.addToSize(-1),  // Shrink sequence so document extends past end.
+        }
+            .addToSize(-1),  // Shrink sequence so document extends past end.
     };
 
     ASSERT_THROWS_CODE(msg.parse(), AssertionException, ErrorCodes::Overflow);
@@ -591,10 +633,23 @@ TEST_F(OpMsgParser, SucceedsWithUnknownOptionalFlags) {
             flags,  //
             kBodySection,
             fromjson("{ping: 1}"),
-        }.parse();
+        }
+            .parse();
     }
 }
 
+TEST_F(OpMsgParser, FailsWithChecksumMismatch) {
+    auto msg = OpMsgBytes{kHaveChecksum,  //
+                          kBodySection,
+                          fromjson("{ping: 1}")}
+                   .appendChecksum(123);
+
+    ASSERT_THROWS_WITH_CHECK(msg.parse(), AssertionException, [](const DBException& ex) {
+        ASSERT_EQ(ex.toStatus().code(), ErrorCodes::ChecksumMismatch);
+        ASSERT(ErrorCodes::isConnectionFatalMessageParseError(ex.toStatus().code()));
+    });
+}
+
 void testSerializer(const Message& fromSerializer, OpMsgBytes&& expected) {
     const auto expectedMsg = expected.done();
     ASSERT_EQ(fromSerializer.operation(), dbMsg);
@@ -861,5 +916,19 @@ TEST(OpMsgRequest, FromDbAndBodyDoesNotCopy) {
     ASSERT_BSONOBJ_EQ(msg.body, fromjson("{ping: 1, $db: 'db'}"));
     ASSERT_EQ(static_cast<const void*>(msg.body.objdata()), bodyPtr);
 }
+
+TEST(OpMsgTest, ChecksumResizesMessage) {
+    auto msg = OpMsgBytes{kNoFlags,  //
+                          kBodySection,
+                          fromjson("{ping: 1}")}
+                   .done();
+
+    // Test that appendChecksum() resizes the buffer if necessary.
+    const auto capacity = msg.sharedBuffer().capacity();
+    OpMsg::appendChecksum(&msg);
+    ASSERT_EQ(msg.sharedBuffer().capacity(), capacity + 4);
+    // The checksum is correct.
+    OpMsg::parse(msg);
+}
 }  // namespace
 }  // namespace mongo
