@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -50,7 +49,8 @@ public:
      *  setShardVersion command will be invoked to initialize the remote shard. Otherwise, the
      *  chunk manager will be used to obtain the shard version to set on the connection.
      */
-    ShardConnection(const ConnectionString& connectionString,
+    ShardConnection(OperationContext* opCtx,
+                    const ConnectionString& connectionString,
                     const std::string& ns,
                     std::shared_ptr<ChunkManager> manager = nullptr);
 
@@ -104,8 +104,9 @@ public:
     }
 
     void donotCheckVersion() {
-        _setVersion = false;
+        invariant(!_finishedInit);
         _finishedInit = true;
+        _setVersion = false;
     }
 
     bool ok() const {
@@ -142,10 +143,12 @@ private:
     const std::string _ns;
     const std::shared_ptr<ChunkManager> _manager;
 
-    bool _finishedInit;
+    bool _finishedInit{false};
+
+    // Specifies whether the call to _finishInit established a version on the connection
+    bool _setVersion{false};
 
     DBClientBase* _conn;
-    bool _setVersion;
 };
 
 /**
