@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -37,7 +36,7 @@
 #include <string>
 
 #include "mongo/bson/util/bson_extract.h"
-#include "mongo/db/auth/internal_user_auth.h"
+#include "mongo/client/authenticate.h"
 #include "mongo/db/client.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/client/version_manager.h"
@@ -58,14 +57,12 @@ void ShardingConnectionHook::onCreate(DBClientBase* conn) {
 
     // Authenticate as the first thing we do
     // NOTE: Replica set authentication allows authentication against *any* online host
-    if (isInternalAuthSet()) {
+    if (auth::isInternalAuthSet()) {
         LOG(2) << "calling onCreate auth for " << conn->toString();
 
-        bool result = conn->authenticateInternalUser();
-
-        uassert(15847,
-                str::stream() << "can't authenticate to server " << conn->getServerAddress(),
-                result);
+        uassertStatusOKWithContext(conn->authenticateInternalUser(),
+                                   str::stream() << "can't authenticate to server "
+                                                 << conn->getServerAddress());
     }
 
     // Delegate the metadata hook logic to the egress hook; use lambdas to pass the arguments in
@@ -102,9 +99,7 @@ void ShardingConnectionHook::onCreate(DBClientBase* conn) {
         uassert(28785,
                 str::stream() << "Unrecognized configsvr mode number: " << configServerModeNumber
                               << ". Range of known configsvr mode numbers is: ["
-                              << minKnownConfigServerMode
-                              << ", "
-                              << maxKnownConfigServerMode
+                              << minKnownConfigServerMode << ", " << maxKnownConfigServerMode
                               << "]",
                 configServerModeNumber >= minKnownConfigServerMode &&
                     configServerModeNumber <= maxKnownConfigServerMode);
