@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -51,13 +50,13 @@ bool containsPattern(const std::string& pattern, const std::string& value) {
 }
 
 #define ASSERT_TEST_FAILS(TEST_STMT) \
-    ASSERT_THROWS(TEST_STMT, mongo::unittest::TestAssertionFailureException)
+    ASSERT_THROWS([&] { TEST_STMT; }(), mongo::unittest::TestAssertionFailureException)
 
-#define ASSERT_TEST_FAILS_MATCH(TEST_STMT, PATTERN)                                       \
-    ASSERT_THROWS_WITH_CHECK(                                                             \
-        TEST_STMT, mongo::unittest::TestAssertionFailureException, ([&](const auto& ex) { \
-            ASSERT_STRING_CONTAINS(ex.getMessage(), (PATTERN));                           \
-        }))
+#define ASSERT_TEST_FAILS_MATCH(TEST_STMT, PATTERN)     \
+    ASSERT_THROWS_WITH_CHECK(                           \
+        [&] { TEST_STMT; }(),                           \
+        mongo::unittest::TestAssertionFailureException, \
+        ([&](const auto& ex) { ASSERT_STRING_CONTAINS(ex.getMessage(), (PATTERN)); }))
 
 TEST(UnitTestSelfTest, DoNothing) {}
 
@@ -270,4 +269,38 @@ TEST(UnitTestSelfTest, StackTraceForAssertion) {
     ASSERT_STRING_CONTAINS(stacktrace, "printStackTrace");
 }
 
+TEST(UnitTestSelfTest, ComparisonAssertionOverloadResolution) {
+    using namespace mongo;
+
+    char xBuf[] = "x";  // Guaranteed different address than "x".
+    const char* x = xBuf;
+
+    // At least one StringData, compare contents:
+    ASSERT_EQ("x"_sd, "x"_sd);
+    ASSERT_EQ("x"_sd, "x");
+    ASSERT_EQ("x"_sd, xBuf);
+    ASSERT_EQ("x"_sd, x);
+    ASSERT_EQ("x", "x"_sd);
+    ASSERT_EQ(xBuf, "x"_sd);
+    ASSERT_EQ(x, "x"_sd);
+
+    // Otherwise, compare pointers:
+    ASSERT_EQ(x, x);
+    ASSERT_EQ(xBuf, xBuf);
+    ASSERT_EQ(x, xBuf);
+    ASSERT_NE("x", xBuf);
+    ASSERT_NE("x", x);
+    ASSERT_NE(xBuf, "x");
+    ASSERT_NE(x, "x");
+}
+
+ASSERT_DOES_NOT_COMPILE(typename Char = char, *std::declval<Char>());
+ASSERT_DOES_NOT_COMPILE(bool B = false, std::enable_if_t<B, int>{});
+
+// Uncomment to check that it fails when it is supposed to. Unfortunately we can't check in a test
+// that this fails when it is supposed to, only that it passes when it should.
+//
+// ASSERT_DOES_NOT_COMPILE(typename Char = char, *std::declval<Char*>());
+// ASSERT_DOES_NOT_COMPILE(bool B = true, std::enable_if_t<B, int>{});
+
 }  // namespace
